@@ -24,7 +24,7 @@
 
 /* global variable */
 //lnf_mem_t *memp;
-int totalrows = 0;
+unsigned long totalrows = 0;
 pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 flist_t *flist;
 int fileidx = 0;
@@ -402,7 +402,7 @@ int main(int argc, char **argv) {
 		output_finish(&output[o]);
 
 		/* header */
-		printf("Input flows: %d, Output flows: %lu\n\n", totalrows, output[o].outputflows);
+		printf("Input flows: %lu, Output flows: %lu\n\n", totalrows, output[o].outputflows);
 
 	}
 
