@@ -335,6 +335,17 @@ int ret;
 				}
 				break;
 			}
+			case LNF_ACL: {
+				lnf_acl_t tacl;
+
+                ret = lnf_rec_fget(lnf_rec, field, (void *)&tacl);
+				if (ret != LNF_OK) {
+					sv = newSV(0);
+				} else {
+					sv = newSVpvn((char *)&tacl, sizeof(lnf_acl_t));
+				}
+				break;
+			}
 			case LNF_STRING: {
 				char buf[LNF_MAX_STRING];
 
@@ -393,6 +404,7 @@ libnf_instance_t *instance;
 
 	instance->field_list = NULL;
 
+
 	return handle;
 }
 
@@ -444,6 +456,7 @@ int i;
 
 int libnf_aggr_add(int handle, int field, int flags, int numbits, int numbits6 ) {
 libnf_instance_t *instance = libnf_instances[handle];
+int ret = 0;
 
 	if (instance == NULL ) {
 		croak("%s handler %d not initialized", NFL_LOG, handle);
@@ -451,16 +464,19 @@ libnf_instance_t *instance = libnf_instances[handle];
 	}
 
 	if (instance->lnf_mem == NULL) {
-		if (lnf_mem_init(&instance->lnf_mem) != LNF_OK ) {
+		if ((ret = lnf_mem_init(&instance->lnf_mem)) != LNF_OK ) {
 			return 0;
 		}
 		/* first and last is always present */
-		if (lnf_mem_fadd(instance->lnf_mem, LNF_FLD_FIRST, LNF_AGGR_MIN, 0, 0) != LNF_OK ) {
-			return 0;
-		}	
-		if (lnf_mem_fadd(instance->lnf_mem, LNF_FLD_LAST, LNF_AGGR_MAX, 0, 0) != LNF_OK ) {
-			return 0;
-		}	
+		/* no need for that - solved in dependency */
+		//if (lnf_mem_fadd(instance->lnf_mem, LNF_FLD_FIRST, LNF_AGGR_MIN, 0, 0) != LNF_OK ) {
+//		if (lnf_mem_fadd(instance->lnf_mem, LNF_FLD_FIRST, 0, numbits, 0) != LNF_OK ) {
+//			return 0;
+//		}	
+		//if (lnf_mem_fadd(instance->lnf_mem, LNF_FLD_LAST, LNF_AGGR_MAX, 0, 0) != LNF_OK ) {
+//		if (lnf_mem_fadd(instance->lnf_mem, LNF_FLD_LAST, 0, numbits, 0) != LNF_OK ) {
+//			return 0;
+//		}	
 	}
 
 	if (lnf_mem_fadd(instance->lnf_mem, field, flags, numbits, numbits6) != LNF_OK ) {
@@ -489,6 +505,25 @@ libnf_instance_t *instance = libnf_instances[handle];
 	return 1;
 } 
 
+int libnf_compatmode(int handle) {
+libnf_instance_t *instance = libnf_instances[handle];
+
+	if (instance == NULL ) {
+		croak("%s handler %d not initialized", NFL_LOG, handle);
+		return 0;
+	}
+
+	if (instance->lnf_mem == NULL) {
+			return 0;
+		}
+
+	if (lnf_mem_setopt(instance->lnf_mem, LNF_OPT_COMP_STATSCMP, NULL, 0) != LNF_OK ) {
+		return 0;
+	}	
+
+	return 1;
+} 
+
 
 int libnf_read_files(int handle, char *filter, int window_start, int window_end, SV *files) {
 libnf_instance_t *instance = libnf_instances[handle];
@@ -866,6 +901,21 @@ lnf_rec_t *lnf_rec;
 				break;
 			}
 
+			case LNF_ACL: {
+				char *s;
+				STRLEN len;
+
+				s = SvPV(sv, len);
+
+				if ( len != sizeof(lnf_acl_t) ) {
+					warn("%s invalid ACL value for %d", NFL_LOG, field);
+					return 0;
+				}
+
+				ret = lnf_rec_fset(lnf_rec, field, (void *)s);
+				break;
+			}
+
 			case LNF_STRING: {
 				char buf[LNF_MAX_STRING];
 				STRLEN len;
