@@ -1,6 +1,5 @@
 /*
- *  Copyright (c) 2016, Peter Haag
- *  Copyright (c) 2014, Peter Haag
+ *  Copyright (c) 2014-2020, Peter Haag
  *  All rights reserved.
  *  
  *  Redistribution and use in source and binary forms, with or without 
@@ -27,8 +26,6 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  Author: peter
- *
  */
 
 #include "config.h"
@@ -40,7 +37,6 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/types.h>
-
 #ifdef HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
 #endif
@@ -49,8 +45,10 @@
 #include <netinet/in.h>
 #endif
 
+#include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/ip.h>
+#include <arpa/inet.h>
 #include <unistd.h>
 #include <stdint.h>
 
@@ -58,25 +56,21 @@
 #include "rbtree.h"
 #include "ipfrag.h"
 
-#ifndef DEVEL
-#   define dbg_printf(...) /* printf(__VA_ARGS__) */
-#else
-#   define dbg_printf(...) printf(__VA_ARGS__)
-#endif
-
 #define KEYLEN (offsetof(IPFragNode_t,data_size) - offsetof(IPFragNode_t, src_addr))
 static int IPFragNodeCMP(struct IPFragNode *e1, struct IPFragNode *e2);
 
-static struct IPFragNode *New_node(void);
+static struct IPFragNode *New_frag_node(void);
 
 static void Free_node(struct IPFragNode *node, int free_data);
 
-static void Remove_node(struct IPFragNode *node);
+static void Remove_node(struct IPFragNode *node, int free_data);
 
 // Insert the IP RB tree code here
 RB_GENERATE(IPFragTree, IPFragNode, entry, IPFragNodeCMP);
 
 static IPFragTree_t *IPFragTree;
+static uint32_t NumFragments;
+static time_t lastExpire = 0;
 
 static int IPFragNodeCMP(struct IPFragNode *e1, struct IPFragNode *e2) {
 uint32_t    *a = &e1->src_addr;
@@ -89,40 +83,40 @@ int i;
  
 } // End of IPFragNodeCMP
 
-static struct IPFragNode *New_node(void) {
+static struct IPFragNode *New_frag_node(void) {
 struct IPFragNode *node;
 
-	node = malloc(sizeof(struct IPFragNode));
+	node = calloc(1, sizeof(struct IPFragNode));
 	if ( !node ) {
 		LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
 		return NULL;
 	}
-	memset((void *)node, 0, sizeof(struct IPFragNode));
 
-	node->data = malloc(IP_MAXPACKET);
+	node->data = calloc(1, IP_MAXPACKET);
 	if ( !node->data ) {
 		LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
 		free(node);
 		return NULL;
 	}
-	memset(node->data, 0, IP_MAXPACKET);
 
 	node->eod = node->data;
 	node->data_size = 0;
 
-	node->holes = malloc(sizeof(hole_t));
+	node->holes = calloc(1, sizeof(hole_t));
 	if ( !node->holes ) {
 		LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
+		free(node->data);
 		free(node);
 		return NULL;
 	}
 	node->holes->next  = NULL;
 	node->holes->first = 0;
 	node->holes->last = IP_MAXPACKET;
+	NumFragments++;
 
 	return node;
 
-} // End of New_node
+} // End of New_frag_node
 
 static void Free_node(struct IPFragNode *node, int free_data) {
 hole_t *hole, *h;
@@ -133,29 +127,31 @@ hole_t *hole, *h;
 		free(hole);
 		hole = h;
 	}
-	if ( free_data) 
+	if (free_data) 
 		free(node->data);
 	free(node);
+	NumFragments--;
 
 } // End of Free_node
 
-static void Remove_node(struct IPFragNode *node) {
+static void Remove_node(struct IPFragNode *node, int free_data) {
 struct IPFragNode *n;
 
 	n = RB_REMOVE(IPFragTree, IPFragTree, node);
 	if ( n ) {
-		Free_node(n, 0);
+		Free_node(n, free_data);
 	} // else - node not in tree
 
 } // End of Remove_node
 
 int IPFragTree_init(void) {
-	IPFragTree = malloc(sizeof(IPFragTree_t));
+	IPFragTree = calloc(1, sizeof(IPFragTree_t));
 	if ( !IPFragTree ) {
 		LogError("malloc() error in %s line %d: %s", __FILE__, __LINE__, strerror(errno) );
 		return 0;
 	}
 	RB_INIT(IPFragTree);
+	NumFragments = 0;
 	dbg_printf("IPFrag key len: %lu\n", KEYLEN);
 	return 1;
 } // End of IPFragTree_init
@@ -163,7 +159,7 @@ int IPFragTree_init(void) {
 void IPFragTree_free(void) {
 struct IPFragNode *node, *nxt;
 
-    // Dump all incomplete flows to the file
+	nxt = NULL;
     for (node = RB_MIN(IPFragTree, IPFragTree); node != NULL; node = nxt) {
         nxt = RB_NEXT(IPFragTree, IPFragTree, node);
         RB_REMOVE(IPFragTree, IPFragTree, node);
@@ -172,28 +168,65 @@ struct IPFragNode *node, *nxt;
 
 	free(IPFragTree);
 	IPFragTree = NULL;
+	NumFragments = 0;
+
 } // End of IPFragTree_free
 
-void *IPFrag_tree_Update(uint32_t src, uint32_t dst, uint32_t ident, uint32_t *length, uint32_t ip_off, void *data) {
+static void IPFragTree_expire(time_t when) {
+struct IPFragNode *node, *nxt;
+
+	uint32_t expireCnt = 0;
+	nxt = NULL;
+    for (node = RB_MIN(IPFragTree, IPFragTree); node != NULL; node = nxt) {
+        nxt = RB_NEXT(IPFragTree, IPFragTree, node);
+		if ( (when - node->last) > 15 ) {
+        	RB_REMOVE(IPFragTree, IPFragTree, node);
+			Free_node(node, 1);
+			expireCnt++;
+		}
+    }
+	dbg_printf("Expired %u incomplete IP fragments, total fragments: %u\n", expireCnt, NumFragments);
+	if ( expireCnt )
+		LogInfo("Expired %u incomplete IP fragments, total fragments: %u", expireCnt, NumFragments);
+
+} // End of IPFragTree_expire
+
+void *IPFrag_tree_Update(time_t when, uint32_t src, uint32_t dst, uint32_t ident, uint32_t *length, uint32_t ip_off, void *data) {
 struct IPFragNode FindNode, *n;
 hole_t *hole, *h, *hole_parent;
 uint16_t more_fragments, first, last, max;
-int found_hole;
+dbg(int found_hole;)
+char src_s[16], dst_s[16];
+
+	if ( (when - lastExpire ) > 10 ) {
+		IPFragTree_expire(when);
+		lastExpire = when;
+	}
+
+#ifdef DEVEL
+	inet_ntop(AF_INET, &src, src_s, 16);
+	inet_ntop(AF_INET, &dst, dst_s, 16);
+	printf("Update %s - %s\n", src_s, dst_s);
+#endif
 
 	FindNode.src_addr = src;
 	FindNode.dst_addr = dst;
 	FindNode.ident 	  = ident;
+	FindNode.last 	  = 0;
+
 	n = RB_FIND(IPFragTree, IPFragTree, &FindNode);
 	if ( !n ) {
-		n = New_node();
+		n = New_frag_node();
 		n->src_addr = src;
 		n->dst_addr = dst;
 		n->ident 	= ident;
+		n->last 	= when;
 		if ( RB_INSERT(IPFragTree, IPFragTree, n) ) {
 			// must never happen
 			LogError("Node insert returned existing node - Software error in %s line %d", __FILE__, __LINE__);
 		}
 	}
+
 	hole = n->holes;
 	hole_parent = NULL;
 
@@ -203,14 +236,14 @@ int found_hole;
 	
 	if ( last > IP_MAXPACKET ) {
 		LogError("Fragment assembly error: last > IP_MAXPACKET");
-		LogError("Fraget assembly: first: %u, last: %u, MF: %u\n", first, last, more_fragments);
+		LogError("Fragment assembly: first: %u, last: %u, MF: %u\n", first, last, more_fragments);
 		return NULL;
 	}
 
 	// last fragment - sets max offset
-	found_hole = 0;
+	dbg(found_hole = 0;)
 	max = more_fragments == 0 ? last : 0;
-	dbg_printf("Fraget assembly: first: %u, last: %u, MF: %u\n", first, last, more_fragments);
+	dbg_printf("Fragment assembly: first: %u, last: %u, MF: %u, ID: %x\n", first, last, more_fragments, ident);
 	while (hole) {
 		uint16_t hole_last;
 		if ( max ) {
@@ -218,16 +251,14 @@ int found_hole;
 			// last fragment offset/length
 			if ( hole->last == IP_MAXPACKET ) {
 				// last fragment has max size
-				if ( max > hole->first ) {
+				if ( max >= hole->first ) {
 					dbg_printf("set max of last fragment: %u\n", max);
 					hole->last = max;
 				} else {
-					LogError("last fragment offset error - teardrop attack??");
-				}
-			} else {
-				// last fragment must always be max offset
-				if ( max < hole->last ) {
-					LogError("last fragment offset error - teardrop attack??");
+					inet_ntop(AF_INET, &src, src_s, 16);
+					inet_ntop(AF_INET, &dst, dst_s, 16);
+					LogError("last fragment offset error - teardrop attack?? SRC: %s, DST: %s",
+							src_s, dst_s);
 				}
 			}
 		}
@@ -246,8 +277,23 @@ int found_hole;
 			continue;
 		}
 
+		// check for overlapping - cut off overlap
+		if ( last > hole->last ) {
+			dbg_printf("Truncate right overlapping fragment: %u -> %u\n", last, hole->last);
+			last = hole->last;
+		}
+
+		if ( first < hole->first ) {
+			dbg_printf("Truncate left overlapping fragment: %u -> %u\n", first, hole->first);
+			first = hole->first;
+		}
+
+		if ( first > last ) {
+			LogInfo("fragment error first %u >= last %u", first, last);
+			return NULL;
+		}
 		// fragment fits into hole
-		found_hole = 1;
+		dbg(found_hole = 1;)
 		if ( last > n->data_size ) 
 			n->data_size = last;
 
@@ -261,7 +307,7 @@ int found_hole;
 				if ( hole_parent ) {
 					hole_parent->next = hole->next;
 				} else {
-					n->holes = NULL;
+					n->holes = hole->next;
 				}
 				free(hole);
 				hole = NULL;
@@ -291,19 +337,31 @@ int found_hole;
 
 		break;
 	}
-	
-	if ( !found_hole )
-		LogError("No space - Fragment overlap: first: %u, last: %u\n", first, last);
+
+#ifdef DEVEL	
+	if ( !found_hole ) {
+	hole_t *h = n->holes;
+		dbg_printf("No space in fragment list for: first: %u, last: %u\n", first, last);
+		while ( h ) {
+			dbg_printf("first: %u,last: %u\n", h->first, h->last);
+			h = h->next;
+		}
+	}
+#endif
 	
 	if ( n->holes == NULL ) {
 		void *data = n->data;
 		n->data_size++;
 		*length = n->data_size;
-		Remove_node(n);
-		dbg_printf("Datagramm complete - size: %u\n", n->data_size);
+		Remove_node(n, 0);
+		dbg_printf("Defragmentation complete - size: %u\n", n->data_size);
 		return data;
 	} else {
 		return NULL;
 	}
 
 } // End of IPFrag_tree_Update
+
+uint32_t IPFragEntries() {
+	return NumFragments;
+} // End of IPFragEntries
