@@ -1,6 +1,5 @@
 /*
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2009, Peter Haag
+ *  Copyright (c) 2009-2020, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -28,17 +27,12 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: nflowcache.c 40 2009-12-16 10:41:44Z haag $
- *
- *  $LastChangedRevision: 40 $
- *	
  */
 
 #include "config.h"
 
 #include <stdio.h>
+#include <stddef.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -53,16 +47,12 @@
 #include <stdint.h>
 #endif
 
+#include "util.h"
+#include "nfdump.h"
 #include "nffile.h"
 #include "nfx.h"
 #include "nflowcache.h"
 
-#ifndef DEVEL
-#   define dbg_printf(...) /* printf(__VA_ARGS__) */
-#else
-#   define dbg_printf(...) printf(__VA_ARGS__)
-#endif
-
 #define ALIGN_BYTES (offsetof (struct { char x; uint64_t y; }, y) - 1)
 
 extern int hash_hit;
@@ -115,6 +105,14 @@ static struct aggregate_info_s {
 	{ "srcip",		{ 8, OffsetSrcIPv6b, 	MaskIPv6, 	 ShiftIPv6 },    	-1, 0, 	NULL	},
 	{ "dstip",		{ 8, OffsetDstIPv6a, 	MaskIPv6, 	 ShiftIPv6 },    	-1, 0,	"%da"	},
 	{ "dstip",		{ 8, OffsetDstIPv6b, 	MaskIPv6, 	 ShiftIPv6 },    	-1, 0, 	NULL	},
+#ifdef NSEL
+	{ "xsrcip",		{ 8, OffsetXLATESRCv6a,	MaskIPv6, 	 ShiftIPv6 },    	-1, 0,	"%xsa" 	},
+	{ "xsrcip",		{ 8, OffsetXLATESRCv6b,	MaskIPv6, 	 ShiftIPv6 },    	-1, 0, 	NULL	},
+	{ "xdstip",		{ 8, OffsetXLATEDSTv6a,	MaskIPv6, 	 ShiftIPv6 },    	-1, 0,	"%xda"	},
+	{ "xdstip",		{ 8, OffsetXLATESRCv6b,	MaskIPv6, 	 ShiftIPv6 },    	-1, 0, 	NULL	},
+	{ "xsrcport",	{ 2, OffsetXLATEPort,   MaskXLATESRCPORT,   ShiftXLATESRCPORT }, 	-1, 0, 	"%xsp"	},
+	{ "xdstport",	{ 2, OffsetXLATEPort,   MaskXLATEDSTPORT,   ShiftXLATEDSTPORT }, 	-1, 0, 	"%xdp"	},
+#endif
 	{ "dstip4",		{ 8, OffsetDstIPv6a, 	MaskIPv6, 	 ShiftIPv6 },     	 0, 0,	"%da"	},
 	{ "dstip4",		{ 8, OffsetDstIPv6b, 	MaskIPv6, 	 ShiftIPv6 },     	 1, 0,	NULL	},
 	{ "dstip6",		{ 8, OffsetDstIPv6a, 	MaskIPv6, 	 ShiftIPv6 },     	 0, 0,	"%da"	},
@@ -910,20 +908,20 @@ Default_key_t *keyptr;
 	} else if ( swap_flow ) {
 		// default 5-tuple aggregation for bidirectional flows
 		keyptr = (Default_key_t *)keymem;
-		keyptr->srcaddr[0]	= flow_record->v6.dstaddr[0];
-		keyptr->srcaddr[1]	= flow_record->v6.dstaddr[1];
-		keyptr->dstaddr[0]	= flow_record->v6.srcaddr[0];
-		keyptr->dstaddr[1]	= flow_record->v6.srcaddr[1];
+		keyptr->srcaddr[0]	= flow_record->V6.dstaddr[0];
+		keyptr->srcaddr[1]	= flow_record->V6.dstaddr[1];
+		keyptr->dstaddr[0]	= flow_record->V6.srcaddr[0];
+		keyptr->dstaddr[1]	= flow_record->V6.srcaddr[1];
 		keyptr->srcport		= flow_record->dstport;
 		keyptr->dstport		= flow_record->srcport;
 		keyptr->proto		= flow_record->prot;
 	} else {
 		// default 5-tuple aggregation
 		keyptr = (Default_key_t *)keymem;
-		keyptr->srcaddr[0]	= flow_record->v6.srcaddr[0];
-		keyptr->srcaddr[1]	= flow_record->v6.srcaddr[1];
-		keyptr->dstaddr[0]	= flow_record->v6.dstaddr[0];
-		keyptr->dstaddr[1]	= flow_record->v6.dstaddr[1];
+		keyptr->srcaddr[0]	= flow_record->V6.srcaddr[0];
+		keyptr->srcaddr[1]	= flow_record->V6.srcaddr[1];
+		keyptr->dstaddr[0]	= flow_record->V6.dstaddr[0];
+		keyptr->dstaddr[1]	= flow_record->V6.dstaddr[1];
 		keyptr->srcport		= flow_record->srcport;
 		keyptr->dstport		= flow_record->dstport;
 		keyptr->proto		= flow_record->prot;
