@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2013-2020, Peter Haag
+ *  Copyright (c) 2013-2021, Peter Haag
  *  All rights reserved.
  *  
  *  Redistribution and use in source and binary forms, with or without 
@@ -54,7 +54,6 @@
 #include <signal.h>
 #include <string.h>
 #include <assert.h>
-// #include <mcheck.h>
 
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
@@ -199,7 +198,7 @@ static void Interrupt_handler(int sig);
 
 static void SetPriv(char *userid, char *groupid );
 
-static pcap_dev_t *setup_pcap_live(char *device, char *filter, int snaplen);
+static pcap_dev_t *setup_pcap_live(char *device, char *filter, int snaplen, int buffer_size);
 
 static pcap_dev_t *setup_pcap_Ffile(FILE *fp, char *filter, int snaplen);
 
@@ -226,6 +225,7 @@ static void usage(char *name) {
 					"-g groupid\tChange group to groupname\n"
 					"-i interface\tread packets from interface\n"
 					"-r pcapfile\tread packets from file\n"
+					"-b num\tset socket buffer size in MB. (default 20MB)\n"
 					"-B num\tset the node cache size. (default 524288)\n"
 					"-s snaplen\tset the snapshot length - default 1526\n"
 					"-e active,inactive\tset the active,inactive flow expire time (s) - default 300,60\n"
@@ -371,7 +371,7 @@ int		err;
 
 } // End of SetPriv
 
-static pcap_dev_t *setup_pcap_live(char *device, char *filter, int snaplen) {
+static pcap_dev_t *setup_pcap_live(char *device, char *filter, int snaplen, int buffer_size) {
 pcap_t 		*handle    = NULL;
 pcap_dev_t	*pcap_dev  = NULL;
 pcap_if_t	*alldevsp = NULL;
@@ -415,14 +415,43 @@ uint32_t	linkoffset, linktype;
 	 *  architectures that support it, you might want tune this value
 	 *  depending on how much traffic you're seeing on the network.
 	 */
-	handle = pcap_open_live(device, snaplen, PROMISC, TIMEOUT, errbuf);
-	if (handle == NULL) {
-		LogError("Couldn't open device %s: %s", device, errbuf);
+	handle = pcap_create(device, errbuf);
+	if ( !handle ) {
+		LogError("pcap_create() failed on %s: %s", device, errbuf);
 		return NULL;
 	}
 
-	// XXX
-	// int pcap_set_buffer_size(pcap_t *p, int buffer_size);
+	if ( pcap_set_snaplen(handle, snaplen)) { 
+		LogError("pcap_set_snaplen() failed: %s", pcap_geterr(handle));
+		pcap_close(handle);
+		return NULL;
+	}
+
+	if ( pcap_set_promisc(handle, PROMISC)) {
+		LogError("pcap_set_promisc() failed: %s", pcap_geterr(handle));
+		pcap_close(handle);
+		return NULL;
+	}
+
+	if ( pcap_set_timeout(handle, TIMEOUT) ) {
+		LogError("pcap_set_promisc() failed: %s", pcap_geterr(handle));
+		pcap_close(handle);
+		return NULL;
+	}
+
+	if ( buffer_size ) 
+		if ( pcap_set_buffer_size(handle, 1024 * 1024 * buffer_size) < 0 ) {
+			LogError("pcap_set_buffer_size() failed: %s", pcap_geterr(handle));
+			pcap_close(handle);
+			return NULL;
+		}
+	// else use platform default
+
+    if ( pcap_activate(handle) ) {
+		LogError("pcap_activate() failed: %s", pcap_geterr(handle));
+		pcap_close(handle);
+		return NULL;
+	}
 
 	if ( filter ) {
 		/* Compile and apply the filter */
@@ -451,6 +480,9 @@ uint32_t	linkoffset, linktype;
 		case DLT_PPP: 
 			linkoffset = 2;
 			break;
+		case DLT_PPP_SERIAL:
+			linkoffset = 4;
+			break;
 		case DLT_NULL: 
 			linkoffset = 4;
 			break;
@@ -568,21 +600,29 @@ uint32_t	linkoffset, linktype;
 } // End of setup_pcap_file
 
 static void SignalThreadTerminate(thread_info_t *thread_info, pthread_cond_t *thread_cond ) {
+struct timespec waitTime;
+
+	waitTime.tv_sec = 0;
+	waitTime.tv_nsec = 10000;
 
 	if ( !thread_info->done ) {
-		dbg_printf("Signal thread[%lu] to terminate\n", (long unsigned)thread_info->tid);
-    	if ( pthread_kill(thread_info->tid, SIGUSR2) != 0 ) {
-			dbg_printf("Failed to signal thread[%lu]\n", (long unsigned)thread_info->tid);
-		} 
-
-		// in case of a condition - signal condition
-		if ( thread_cond ) 
-			pthread_cond_signal(thread_cond);
+		do {
+			dbg_printf("Signal thread[%lu] to terminate\n", (long unsigned)thread_info->tid);
+    		if ( pthread_kill(thread_info->tid, SIGUSR2) != 0 ) {
+				dbg_printf("Failed to signal thread[%lu]\n", (long unsigned)thread_info->tid);
+			} 
+			nanosleep(&waitTime, NULL);
+		} while ( !thread_info->done );
 
 	} else {
 		dbg_printf("thread[%lu] gone already\n", (long unsigned)thread_info->tid);
 	}
 
+	// in case of a condition - signal condition
+	if ( thread_cond ) {
+		pthread_cond_signal(thread_cond);
+	}
+
    	if( pthread_join(thread_info->tid, NULL) == 0 ) {
        	dbg_printf("thread %lu joined\n", (long unsigned)thread_info->tid);
 	} else {
@@ -866,7 +906,7 @@ int runs = 0;
 			pcapfile->alternate_size = 0;
 		}
 
-		// if we are done, try to flsuh main data buffer
+		// if we are done, try to flush main data buffer
 		if ( args->done && pcapfile->data_size ) {
 			dbg_printf("Done: Flush all buffers\n");
 			// flush alternate buffer
@@ -1191,7 +1231,7 @@ int main(int argc, char *argv[]) {
 sigset_t			signal_set;
 struct sigaction	sa;
 int c, snaplen, err, do_daemonize;
-int subdir_index, compress, expire, cache_size;
+int subdir_index, compress, expire, cache_size, buff_size;
 int active, inactive;
 FlowSource_t	*fs;
 dirstat_t 		*dirstat;
@@ -1203,12 +1243,12 @@ pcap_dev_t 		*pcap_dev;
 p_packet_thread_args_t *p_packet_thread_args;
 p_flow_thread_args_t *p_flow_thread_args;
 
-	snaplen			= 1526;
+	snaplen			= 1500;
 	do_daemonize	= 0;
 	launcher_pid	= 0;
 	device			= NULL;
 	pcapfile		= NULL;
-	filter			= NULL;
+	filter			= "ip";
 	pidfile[0]		= '\0';
 	t_win			= TIME_WINDOW;
 	datadir			= DEFAULT_DIR;
@@ -1223,9 +1263,10 @@ p_flow_thread_args_t *p_flow_thread_args;
 	verbose			= 0;
 	expire			= 0;
 	cache_size		= 0;
+	buff_size		= 20;
 	active			= 0;
 	inactive		= 0;
-	while ((c = getopt(argc, argv, "B:DEI:e:g:hi:j:r:s:l:p:P:t:u:S:T:Vyz")) != EOF) {
+	while ((c = getopt(argc, argv, "B:DEI:b:e:g:hi:j:r:s:l:p:P:t:u:S:T:Vyz")) != EOF) {
 		switch (c) {
 			struct stat fstat;
 			case 'h':
@@ -1251,6 +1292,13 @@ p_flow_thread_args_t *p_flow_thread_args;
 			case 'I':
 				Ident = strdup(optarg);
 				break;
+			case 'b':
+				buff_size = atoi(optarg);
+				if (buff_size <= 0 || buff_size > 2047 ) {
+					LogError("ERROR: Buffer size in MB must be betwee 0..2047 (2GB max, default 20M)");
+					exit(EXIT_FAILURE);
+				}
+				break;
 			case 'i':
 				device = optarg;
 				break;
@@ -1419,7 +1467,7 @@ p_flow_thread_args_t *p_flow_thread_args;
 	if ( pcapfile ) {
 		pcap_dev = setup_pcap_file(pcapfile, filter, snaplen);
 	} else {
-		pcap_dev = setup_pcap_live(device, filter, snaplen);
+		pcap_dev = setup_pcap_live(device, filter, snaplen, buff_size);
 	}
 	if (!pcap_dev) {
 		exit(EXIT_FAILURE);
