@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2020, Peter Haag
+ *  Copyright (c) 2009-2021, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -59,27 +59,27 @@ typedef void (*string_function_t)(master_record_t *, char *);
 static struct token_list_s {
 	string_function_t	string_function;	// function generation output string
 	char				*string_buffer;		// buffer for output string
-} *token_list;
+} *token_list = NULL;
 
 static int	max_token_index	= 0;
 static int	token_index		= 0;
 
 #define BLOCK_SIZE	32
 
-static char **format_list;		// ordered list of all individual strings formating the output line
+static char **format_list = NULL;		// ordered list of all individual strings formating the output line
 static int	max_format_index	= 0;
 static int	format_index		= 0;
 
 static int		do_tag 		 = 0;
 static int 		long_v6 	 = 0;
 static int		printPlain	 = 0;
-static double	duration;
+static double	duration	 = 0;
 
 #define STRINGSIZE 10240
 #define IP_STRING_LEN (INET6_ADDRSTRLEN)
 
-static char header_string[STRINGSIZE];
-static char data_string[STRINGSIZE];
+static char header_string[STRINGSIZE] = {0};
+static char data_string[STRINGSIZE] = {0};
 
 // tag 
 static char tag_string[2];
@@ -95,6 +95,8 @@ static void AddString(char *string);
 
 static void String_FlowFlags(master_record_t *r, char *string);
 
+static void String_Version(master_record_t *r, char *string);
+
 static void String_FirstSeen(master_record_t *r, char *string);
 
 static void String_LastSeen(master_record_t *r, char *string);
@@ -177,6 +179,10 @@ static void String_DstVlan(master_record_t *r, char *string);
 
 static void String_FwdStatus(master_record_t *r, char *string);
 
+static void String_BiFlowDir(master_record_t *r, char *string);
+
+static void String_FlowEndReason(master_record_t *r, char *string);
+
 static void String_Flags(master_record_t *r, char *string);
 
 static void String_InSrcMac(master_record_t *r, char *string);
@@ -279,6 +285,7 @@ static struct format_token_list_s {
 	string_function_t	string_function;	// function generation output string
 } format_token_list[] = {
 	{ "%ff", 0, "Flow Flags", 				String_FlowFlags }, 	// flow flags in hex
+	{ "%nfv", 0, "Ver", 					String_Version }, 		// netflow version
 	{ "%tfs", 0, "Date first seen        ", String_FirstSeen },		// Start Time - first seen
 	{ "%ts",  0, "Date first seen        ", String_FirstSeen },		// Start Time - first seen
 	{ "%tsr",  0, "Date first seen (raw)    ", String_FirstSeenRaw },		// Start Time - first seen, seconds
@@ -323,6 +330,8 @@ static struct format_token_list_s {
 	{ "%smk", 0, "SMask", 				 	String_SrcMask },		// Src mask
 	{ "%dmk", 0, "DMask", 				 	String_DstMask },		// Dst mask
 	{ "%fwd", 0, "Fwd", 				 	String_FwdStatus },		// Forwarding Status
+	{ "%bfd", 0, "Bfd", 				 	String_BiFlowDir },		// BiFlow Direction
+	{ "%end", 0, "End", 				 	String_FlowEndReason },	// Flow End Reason
 	{ "%svln", 0, "SVlan", 				 	String_SrcVlan },		// Src Vlan
 	{ "%dvln", 0, "DVlan", 				 	String_DstVlan },		// Dst Vlan
 	{ "%ismc", 0, "  In src MAC Addr", 	 	String_InSrcMac },		// Input Src Mac Addr
@@ -439,11 +448,13 @@ int	i, index;
 
 } // End of format_special 
 
-void text_prolog(void) {
+void text_prolog(bool quiet) {
+	if ( quiet )
+		return;
 	printf("%s\n", header_string);
 } // End of text_prolog
 
-void text_epilog(void) {
+void text_epilog(bool quiet) {
 	// empty
 } // End of text_epilog
 
@@ -567,9 +578,9 @@ int	i, remaining;
 					if ( strncmp(format_token_list[i].token, c, len) == 0 ) {	// token found
 						AddToken(i);
 						if ( long_v6 && format_token_list[i].is_address )
-							snprintf(h, STRINGSIZE-1-strlen(h), "%23s%s", "", format_token_list[i].header);
+							snprintf(h, STRINGSIZE-1-strlen(header_string), "%23s%s", "", format_token_list[i].header);
 						else
-							snprintf(h, STRINGSIZE-1-strlen(h), "%s", format_token_list[i].header);
+							snprintf(h, STRINGSIZE-1-strlen(header_string), "%s", format_token_list[i].header);
 						h += strlen(h);
 						c[len] = p;
 						c += len;
@@ -588,23 +599,23 @@ int	i, remaining;
 		} else {			// it's a static string
 			/* a static string goes up to next '%' or end of string */
 			char *p = strchr(c, '%');
-			char format[32];
+			char printFormat[32];
 			if ( p ) {
 				// p points to next '%' token
 				*p = '\0';
 				AddString(strdup(c));
-				snprintf(format, 31, "%%%zus", strlen(c));
-				format[31] = '\0';
-				snprintf(h, STRINGSIZE-1-strlen(h), format, "");
+				snprintf(printFormat, 31, "%%%zus", strlen(c));
+				printFormat[31] = '\0';
+				snprintf(h, STRINGSIZE-1-strlen(header_string), printFormat, "");
 				h += strlen(h);
 				*p = '%';
 				c = p;
 			} else {
 				// static string up to end of format string
 				AddString(strdup(c));
-				snprintf(format, 31, "%%%zus", strlen(c));
-				format[31] = '\0';
-				snprintf(h, STRINGSIZE-1-strlen(h), format, "");
+				snprintf(printFormat, 31, "%%%zus", strlen(c));
+				printFormat[31] = '\0';
+				snprintf(h, STRINGSIZE-1-strlen(header_string), printFormat, "");
 				h += strlen(h);
 				*c = '\0';
 			}
@@ -638,7 +649,32 @@ static void String_FlowFlags(master_record_t *r, char *string) {
 	string[MAX_STRING_LENGTH-1] = '\0';
 
 } // End of String_FlowFlags
- 
+
+static void String_Version(master_record_t *r, char *string) {
+
+	char *type;
+	if ( TestFlag(r->flags, FLAG_EVENT) ) {
+		type = "EVT";
+		snprintf(string, MAX_STRING_LENGTH-1, "%s%u", type, r->nfversion);
+	} else {
+		if ( r->nfversion != 0 ) {
+			if ( r->nfversion & 0x80 ) {
+				type = "Sv";
+			} else if ( r->nfversion & 0x40 ) {
+				type = "Pv";
+			} else {
+				type = "Nv";
+			}
+			snprintf(string, MAX_STRING_LENGTH-1, "%s%u", type, r->nfversion & 0x0F);
+		} else {
+			// compat with previous versions
+			type = "FLO";
+			snprintf(string, MAX_STRING_LENGTH-1, "%s", type);
+		}
+	}
+
+} // End of String_Version
+
 static void String_FirstSeen(master_record_t *r, char *string) {
 time_t 	tt;
 struct tm * ts;
@@ -1185,9 +1221,23 @@ static void String_FwdStatus(master_record_t *r, char *string) {
 
 } // End of String_FwdStatus
 
+static void String_BiFlowDir(master_record_t *r, char *string) {
+
+	snprintf(string, MAX_STRING_LENGTH-1 ,"%3u", r->biFlowDir);
+	string[MAX_STRING_LENGTH-1] = '\0';
+
+} // End of String_BiFlowDir
+
+static void String_FlowEndReason(master_record_t *r, char *string) {
+
+	snprintf(string, MAX_STRING_LENGTH-1 ,"%3u", r->flowEndReason);
+	string[MAX_STRING_LENGTH-1] = '\0';
+
+} // End of String_FlowEndReason
+
 static void String_Flags(master_record_t *r, char *string) {
 
-	snprintf(string, MAX_STRING_LENGTH-1 ,"%8s", FlagsString(r->tcp_flags));
+	snprintf(string, MAX_STRING_LENGTH-1 ,"%8s", FlagsString(r->prot == IPPROTO_TCP ? r->tcp_flags :0));
 	string[MAX_STRING_LENGTH-1] = '\0';
 
 } // End of String_Flags
