@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2019-2020, Peter Haag
+ *  Copyright (c) 2019-2021, Peter Haag
  *  All rights reserved.
  *  
  *  Redistribution and use in source and binary forms, with or without 
@@ -58,12 +58,12 @@ static char data_string[STRINGSIZE];
 // record counter 
 static uint32_t recordCount;
 
-void raw_prolog(void) {
+void raw_prolog(bool quiet) {
 	recordCount = 0;
 	memset(data_string, 0, STRINGSIZE);
 } // End of raw_prolog
 
-void raw_epilog(void) {
+void raw_epilog(bool quiet) {
 	// empty
 } // End of raw_epilog
 
@@ -181,12 +181,16 @@ extension_map_t	*extension_map = r->map_ref;
 	snprintf(_s, slen-1,
 "  fwd status   =               %3u\n"
 "  tcp flags    =              0x%.2x %s\n"
+"  biFlow Dir   =              0x%.2x %s\n"
+"  end reason   =              0x%.2x %s\n"
 "  proto        =               %3u %s\n"
 "  (src)tos     =               %3u\n"
 "  (in)packets  =        %10llu\n"
 "  (in)bytes    =        %10llu\n",
-	r->fwd_status, r->tcp_flags, FlagsString(r->tcp_flags), r->prot, ProtoString(r->prot, 0), r->tos,
-		(unsigned long long)r->dPkts, (unsigned long long)r->dOctets);
+	r->fwd_status, r->prot == IPPROTO_TCP ? r->tcp_flags : 0, FlagsString(r->prot == IPPROTO_TCP ? r->tcp_flags :0), 
+	r->biFlowDir, biFlowString(r->biFlowDir), 
+	r->flowEndReason, FlowEndString(r->flowEndReason), r->prot, 
+	ProtoString(r->prot, 0), r->tos, (unsigned long long)r->dPkts, (unsigned long long)r->dOctets);
 
 	_slen = strlen(data_string);
 	_s = data_string + _slen;
@@ -360,12 +364,14 @@ extension_map_t	*extension_map = r->map_ref;
 			case EX_MPLS: {
 				unsigned int i;
 				for ( i=0; i<10; i++ ) {
-					snprintf(_s, slen-1,
+					if (r->mpls_label[i] != 0) {
+						snprintf(_s, slen-1,
 "  MPLS Lbl %2u  =      %8u-%1u-%1u\n", i+1
 , r->mpls_label[i] >> 4 , (r->mpls_label[i] & 0xF ) >> 1, r->mpls_label[i] & 1 );
-					_slen = strlen(data_string);
-					_s = data_string + _slen;
-					slen = STRINGSIZE - _slen;
+						_slen = strlen(data_string);
+						_s = data_string + _slen;
+						slen = STRINGSIZE - _slen;
+					}
 				}
 			} break;
 			case EX_ROUTER_IP_v4:
