@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2014-2020, Peter Haag
+ *  Copyright (c) 2014-2021, Peter Haag
  *  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
@@ -71,7 +71,6 @@
 #include "flowtree.h"
 #include "ipfrag.h"
 #include "pcaproc.h"
-#include "content_dns.h"
 #include "netflow_pcap.h"
 
 struct pcap_timeval {
@@ -205,13 +204,6 @@ void PcapDump(pcapfile_t *pcapfile,  struct pcap_pkthdr *h, const u_char *sp) {
 struct pcap_sf_pkthdr sf_hdr;
 size_t	size = sizeof(struct pcap_sf_pkthdr) + h->caplen;
 
-/*
-	if ( pcapfile->pd)
-		pcap_dump((u_char *)pcapfile->pd, h, sp);
-	else
-		printf("NULL handle\n");
-	return;
-*/
 	if ( (pcapfile->data_size + size ) > BUFFSIZE ) {
 		void *_b;
 		// no space left in buffer - rotate buffers
@@ -345,16 +337,32 @@ static inline void ProcessICMPFlow(FlowSource_t	*fs, struct FlowNode *NewNode )
 } // End of ProcessICMPFlow
 
 static inline void ProcessOtherFlow(FlowSource_t *fs, struct FlowNode *NewNode ) {
+struct FlowNode *Node;
 
-	// Flush Other packets directly
-	StorePcapFlow(fs, NewNode);
-	dbg_printf("Flush Other flow: Proto: %u, Packets: %u, Bytes: %u\n", NewNode->proto, NewNode->packets, NewNode->bytes);
+	assert(NewNode->memflag == NODE_IN_USE);
+
+	// insert ESP/AH traffic
+	Node = Insert_Node(NewNode);
+	// if insert fails, the existing node is returned -> flow exists already
+	if ( Node == NULL ) {
+		dbg_printf("New flow IP proto: %u. Packets: %u, Bytes: %u\n",
+			NewNode->proto, NewNode->packets, NewNode->bytes);
+		return;
+	}
+	assert(Node->memflag == NODE_IN_USE);
+
+	// update existing flow
+	Node->packets++;
+	Node->bytes += NewNode->bytes;
+	Node->t_last = NewNode->t_last;
+	dbg_printf("Existing flow IP proto: %u Packets: %u, Bytes: %u\n",
+		NewNode->proto, Node->packets, Node->bytes);
 
 	Free_Node(NewNode);
 
-
 } // End of ProcessOtherFlow
 
+
 void ProcessFlowNode(FlowSource_t *fs, struct FlowNode *node) {
 
 	switch (node->proto) {
@@ -379,7 +387,7 @@ struct FlowNode	*Node;
 struct ip 	  *ip;
 void		  *payload, *defragmented;
 uint32_t	  size_ip, offset, data_len, payload_len, bytes;
-uint16_t	  version, ethertype, proto;
+uint16_t	  version, IPproto;
 char		  s1[64];
 char		  s2[64];
 static unsigned pkg_cnt = 0;
@@ -390,83 +398,129 @@ static unsigned pkg_cnt = 0;
 	pcap_dev->proc_stat.packets++;
 	offset = pcap_dev->linkoffset;
 	defragmented = NULL;
+	Node = NULL;
 
-	Node = New_Node();
-	if ( !Node ) {
-		pcap_dev->proc_stat.skipped++;
-		LogError("Node allocation error - skip packet");
-		return;
-	}
-
-	if ( pcap_dev->linktype == DLT_EN10MB ) {
-		ethertype = data[12] << 0x08 | data[13];
-		int	IEEE802 = ethertype <= 1500;
-		if ( IEEE802 ) {
-			pcap_dev->proc_stat.skipped++;
-			Free_Node(Node);
-			return;
-		}
-		REDO_LINK:
-			switch (ethertype) {
-				case 0x800:	 // IPv4
-				case 0x86DD: // IPv6
+	uint16_t protocol = 0;
+	switch (pcap_dev->linktype) {
+		case DLT_EN10MB:
+			protocol = data[12] << 0x08 | data[13];
+			int	IEEE802 = protocol <= 1500;
+			if ( IEEE802 ) {
+				pcap_dev->proc_stat.skipped++;
+				return;
+			}
+			break;
+		case DLT_RAW:
+			protocol = 0x800;
+			break;
+		case DLT_PPP: 
+			protocol = 0x800;
+			break;
+		case DLT_PPP_SERIAL:
+			protocol = 0x800;
+			break;
+		case DLT_LOOP: 
+		case DLT_NULL: {
+			uint32_t header;
+			if ( pcap_dev->linktype == DLT_LOOP ) 
+				header = ntohl(*((uint32_t *)data));
+			else
+				header = *((uint32_t *)data);
+			switch (header) {
+				case 2:
+					protocol = 0x800;
 					break;
-				case 0x8100: {	// VLAN
-					do {
-						vlan_hdr_t *vlan_hdr = (vlan_hdr_t *)(data + offset);  // offset points to end of link layer
-						dbg_printf("VLAN ID: %u, type: 0x%x\n",
-							ntohs(vlan_hdr->vlan_id), ntohs(vlan_hdr->type) );
-						ethertype = ntohs(vlan_hdr->type);
-/*
-pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
-	  pkt->vlans[pkt->vlan_count].cfi = (p[0] >> 4) & 1;
-	  pkt->vlans[pkt->vlan_count].vid = uint_16_be(p) & 0xfff;
-*/
-						offset += 4;
-					} while ( ethertype == 0x8100 );
-			
-					// redo ethertype evaluation
-					goto REDO_LINK;
-					} break;
-				case 0x26:	 // ?? multicast router termination ??
-				case 0x32:	
-				case 0x806:	 // skip ARP
-				case 0x4305: // B.A.T.M.A.N. BATADV
-				case 0x886f: // MS NLB heartbeat
-				case 0x88a2: // ATA over ethernet
-				case 0x88cc: // CISCO LLDP
-				case 0x9000: // Loop
-				case 0x9003: 
-				case 0x880b: // PPP - rfc 7042
-				case 0x6558: // Ethernet Bridge
-					pcap_dev->proc_stat.skipped++;
-					Free_Node(Node);
-					goto END_FUNC;
+				case 24:
+				case 28:
+				case 30:
+					protocol = 0x86DD;
 					break;
 				default:
-					pcap_dev->proc_stat.unknown++;
-					LogInfo("Unsupported ether type: 0x%x, packet: %u", ethertype, pkg_cnt);
-					Free_Node(Node);
-					goto END_FUNC;
+					LogInfo("Unsupported DLT_NULL protocol: 0x%x, packet: %u", header, pkg_cnt);
+				return;
 			}
-	} else if ( pcap_dev->linktype != DLT_RAW ) { // we can still process raw IP
-		LogInfo("Unsupported link type: 0x%x, packet: %u", pcap_dev->linktype, pkg_cnt);
-		Free_Node(Node);
-		return;
+			} break;
+#ifndef DLT_LINUX_SLL
+#define DLT_LINUX_SLL 113
+#endif
+		case DLT_LINUX_SLL: 
+			protocol = data[14] << 8 | data[15];
+			break;
+		case DLT_IEEE802_11: 
+			protocol = 0x800;
+			break;
+		default:
+			LogInfo("Unsupported link type: 0x%x, packet: %u", pcap_dev->linktype, pkg_cnt);
+			return;
 	}
 
+	REDO_LINK:
+		switch (protocol) {
+			case 0x800:	 // IPv4
+			case 0x86DD: // IPv6
+				break;
+			case 0x8100: {	// VLAN
+				do {
+					vlan_hdr_t *vlan_hdr = (vlan_hdr_t *)(data + offset);  // offset points to end of link layer
+					dbg_printf("VLAN ID: %u, type: 0x%x\n",
+						ntohs(vlan_hdr->vlan_id), ntohs(vlan_hdr->type) );
+					protocol = ntohs(vlan_hdr->type);
+					offset += 4;
+				} while ( protocol == 0x8100 );
+		
+				// redo protocol evaluation
+				goto REDO_LINK;
+				} break;
+			case 0x8847: { // MPLS
+				// unwind MPLS label stack
+				uint32_t *mpls = (uint32_t *)(data + offset);
+				offset += 4;
+				dbg_printf("MPLS label: %x\n", ntohl(*mpls) >> 8);
+				while ((offset < hdr->caplen) && ((ntohl(*mpls) & 0x100) == 0)) { // check for Bottom of stack
+					offset += 4;
+					mpls++;
+					dbg_printf("MPLS label: %x\n", ntohl(*mpls) >> 8);
+				}
+				uint8_t *hdr = (uint8_t *)data + offset;
+				if((*hdr >> 4) == 4)
+					protocol = 0x0800;	// IPv4
+				else if((*hdr >> 4) == 6)
+					protocol = 0x86DD;	// IPv6
+				else {
+					LogInfo("Unsupported protocol: 0x%x", *hdr >> 4);
+					goto END_FUNC;
+				}
+
+				// redo protocol evaluation
+				goto REDO_LINK;
+				} break;
+			case 0x26:	 // ?? multicast router termination ??
+			case 0x32:	
+			case 0x806:	 // skip ARP
+			case 0x4305: // B.A.T.M.A.N. BATADV
+			case 0x886f: // MS NLB heartbeat
+			case 0x88a2: // ATA over ethernet
+			case 0x88cc: // CISCO LLDP
+			case 0x9000: // Loop
+			case 0x9003: 
+			case 0x8808: // Ethernet flow control
+			case 0x880b: // PPP - rfc 7042
+			case 0x6558: // Ethernet Bridge
+				pcap_dev->proc_stat.skipped++;
+				goto END_FUNC;
+				break;
+			default:
+				pcap_dev->proc_stat.unknown++;
+				LogInfo("Unsupported ether type: 0x%x, packet: %u", protocol, pkg_cnt);
+				goto END_FUNC;
+		}
+
 	if (hdr->caplen < offset) {
 		pcap_dev->proc_stat.short_snap++;
 		LogInfo("Short packet: %u/%u", hdr->caplen, offset);
-		Free_Node(Node);
 		goto END_FUNC;
 	}
 
-	Node->t_first.tv_sec = hdr->ts.tv_sec;
-	Node->t_first.tv_usec = hdr->ts.tv_usec;
-	Node->t_last.tv_sec  = hdr->ts.tv_sec;
-	Node->t_last.tv_usec  = hdr->ts.tv_usec;
-
 	data	 = data + offset;
 	data_len = hdr->caplen - offset;
 	offset	 = 0;
@@ -479,13 +533,13 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 		// REDO loop could result in a memory leak, if again IP is fragmented
 		// XXX memory leak to be fixed
 		LogError("Fragmentation memory leak triggered!");
+		goto END_FUNC;
 	}
 
 	ip  	= (struct ip *)(data + offset); // offset points to end of link layer
 	version = ip->ip_v;	 // ip version
 
 	if ( version == 6 ) {
-		uint64_t *addr;
 		struct ip6_hdr *ip6 = (struct ip6_hdr *) (data + offset);
 		size_ip = sizeof(struct ip6_hdr);
 		offset = size_ip;	// offset point to end of IP header
@@ -494,12 +548,11 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			LogInfo("Packet: %u Length error: data_len: %u < size IPV6: %u, captured: %u, hdr len: %u",
 				pkg_cnt, data_len, size_ip, hdr->caplen, hdr->len);	
 			pcap_dev->proc_stat.short_snap++;
-			Free_Node(Node);
 			goto END_FUNC;
 		}
 
 		// XXX Extension headers not processed
-		proto		= ip6->ip6_ctlun.ip6_un1.ip6_un1_nxt;
+		IPproto		= ip6->ip6_ctlun.ip6_un1.ip6_un1_nxt;
 		payload_len = bytes = ntohs(ip6->ip6_ctlun.ip6_un1.ip6_un1_plen);
 
 		if (data_len < (payload_len + size_ip) ) {
@@ -513,11 +566,25 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 
 		payload = (void *)ip + size_ip;
 
-		addr = (uint64_t *)&ip6->ip6_src;
+		Node = New_Node();
+		if ( !Node ) {
+			pcap_dev->proc_stat.skipped++;
+			LogError("Node allocation error - skip packet");
+			return;
+		}
+		Node->t_first.tv_sec = hdr->ts.tv_sec;
+		Node->t_first.tv_usec = hdr->ts.tv_usec;
+		Node->t_last.tv_sec  = hdr->ts.tv_sec;
+		Node->t_last.tv_usec  = hdr->ts.tv_usec;
+
+		// keep compiler happy - get's optimized out anyway
+		void *p = (void *)&ip6->ip6_src;
+		uint64_t *addr = (uint64_t *)p;
 		Node->src_addr.v6[0] = ntohll(addr[0]);
 		Node->src_addr.v6[1] = ntohll(addr[1]);
 
-		addr = (uint64_t *)&ip6->ip6_dst;
+		p = (void *)&ip6->ip6_dst;
+		addr = (uint64_t *)p;
 		Node->dst_addr.v6[0] = ntohll(addr[0]);
 		Node->dst_addr.v6[1] = ntohll(addr[1]);
 		Node->version = AF_INET6;
@@ -529,10 +596,9 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 		offset = size_ip;	// offset point to end of IP header
 
 		if ( data_len < size_ip ) {
-			LogInfo("Packet: %u Length error: data_len: %u < size IPV4: %u, captured: %u, hdr len: %u",
+			dbg_printf("Packet: %u Length error: data_len: %u < size IPV4: %u, captured: %u, hdr len: %u\n",
 				pkg_cnt, data_len, size_ip, hdr->caplen, hdr->len);	
 			pcap_dev->proc_stat.short_snap++;
-			Free_Node(Node);
 			goto END_FUNC;
 		}
 
@@ -541,7 +607,7 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 
 		payload_len -= size_ip;	// ajust length compatibel IPv6
 		payload = (void *)ip + size_ip;
-		proto   = ip->ip_p;
+		IPproto   = ip->ip_p;
 
 		if (data_len < (payload_len + size_ip) ) {
 			// capture len was limited - so adapt payload_len
@@ -573,7 +639,6 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			if ( defragmented == NULL ) {
 				// not yet complete
 				dbg_printf("Fragmentation not yet completed. Size %u bytes\n", payload_len);
-				Free_Node(Node);
 				goto END_FUNC;
 			}
 			dbg_printf("Fragmentation complete\n");
@@ -582,6 +647,17 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 		}
 		bytes 		= payload_len;
 
+		Node = New_Node();
+		if ( !Node ) {
+			pcap_dev->proc_stat.skipped++;
+			LogError("Node allocation error - skip packet");
+			return;
+		}
+		Node->t_first.tv_sec = hdr->ts.tv_sec;
+		Node->t_first.tv_usec = hdr->ts.tv_usec;
+		Node->t_last.tv_sec  = hdr->ts.tv_sec;
+		Node->t_last.tv_usec  = hdr->ts.tv_usec;
+
 		Node->src_addr.v6[0] = 0;
 		Node->src_addr.v6[1] = 0;
 		Node->src_addr.v4 = ntohl(ip->ip_src.s_addr);
@@ -593,17 +669,16 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 	} else {
 		LogInfo("ProcessPacket() Unsupported protocol version: %i", version);
 		pcap_dev->proc_stat.unknown++;
-		Free_Node(Node);
 		goto END_FUNC;
 	}
 
 	Node->packets = 1;
 	Node->bytes   = bytes;
-	Node->proto   = proto;
+	Node->proto   = IPproto;
 	dbg_printf("Payload: %u bytes, Full packet: %u bytes\n", payload_len, bytes);
 
 	// TCP/UDP decoding
-	switch (proto) {
+	switch (IPproto) {
 		case IPPROTO_UDP: {
 			struct udphdr *udp = (struct udphdr *)payload;
 			uint16_t UDPlen = ntohs(udp->uh_ulen);
@@ -615,31 +690,17 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 				Free_Node(Node);
 				break;
 			}
-			uint32_t size_udp_payload = ntohs(udp->uh_ulen) - 8;
 
-			if ( (bytes == payload_len ) && (payload_len - sizeof(struct udphdr)) < size_udp_payload ) {
-				LogInfo("UDP payload length error: Expected %u, have %u bytes, SRC %s, DST %s",
-					size_udp_payload, (payload_len - (unsigned)sizeof(struct udphdr)),
-					inet_ntop(AF_INET, &ip->ip_src, s1, sizeof(s1)),
-					inet_ntop(AF_INET, &ip->ip_dst, s2, sizeof(s2)));
-				Free_Node(Node);
-				break;
-			}
 			payload = payload + sizeof(struct udphdr);
 			payload_len -= sizeof(struct udphdr);
 			dbg_printf("UDP: size: %u, SRC: %i, DST: %i\n",
-				size_udp_payload, ntohs(udp->uh_sport), ntohs(udp->uh_dport));
+				payload_len, ntohs(udp->uh_sport), ntohs(udp->uh_dport));
 
 			Node->bytes = payload_len;
 			Node->flags = 0;
 			Node->src_port = ntohs(udp->uh_sport);
 			Node->dst_port = ntohs(udp->uh_dport);
 
-			if ( hdr->caplen == hdr->len ) {
-				// process payload of full packets
-				if ( (bytes == payload_len) && (Node->src_port == 53 || Node->dst_port == 53) )
- 					content_decode_dns(Node, payload, payload_len);
-			}
 			Push_Node(NodeList, Node);
 			} break;
 		case IPPROTO_TCP: {
@@ -648,12 +709,6 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			size_tcp = tcp->th_off << 2;
 
 			if ( payload_len < size_tcp ) {
-				LogInfo("TCP header length error: len: %u < size TCP header: %u, SRC %s, DST %s",
-					payload_len, size_tcp,
-					inet_ntop(AF_INET, &ip->ip_src, s1, sizeof(s1)),
-					inet_ntop(AF_INET, &ip->ip_dst, s2, sizeof(s2)));
-                pcap_dev->proc_stat.short_snap++;
-
 				pcap_dev->proc_stat.short_snap++;
 				Free_Node(Node);
 				break;
@@ -702,8 +757,6 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			uint32_t size_inner_ip = sizeof(struct ip6_hdr);
 
 			if ( payload_len < size_inner_ip ) {
-				LogInfo("IPIPv6 tunnel header length error: len: %u < size inner IP: %u",
-					payload_len, size_inner_ip);	
 				pcap_dev->proc_stat.short_snap++;
 				Free_Node(Node);
 				goto END_FUNC;
@@ -727,8 +780,6 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			uint32_t size_inner_ip = (inner_ip->ip_hl << 2);
 
 			if ( payload_len < size_inner_ip ) {
-				LogInfo("IPIP tunnel header length error: len: %u < size inner IP: %u",
-					payload_len, size_inner_ip);	
 				pcap_dev->proc_stat.short_snap++;
 				Free_Node(Node);
 				break;
@@ -753,15 +804,13 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 			uint32_t gre_hdr_size = sizeof(gre_hdr_t); // offset points to end of inner IP
 
 			if ( payload_len < gre_hdr_size ) {
-				LogError("GRE tunnel header length error: len: %u < size GRE hdr: %u",
-					payload_len, gre_hdr_size);	
 				pcap_dev->proc_stat.short_snap++;
 				Free_Node(Node);
 				break;
 			}
 
-			dbg_printf("GRE proto encapsulation: type: 0x%x\n", ethertype);
-			ethertype = ntohs(gre_hdr->type);
+			dbg_printf("GRE proto encapsulation: type: 0x%x\n", protocol);
+			protocol = ntohs(gre_hdr->type);
 			offset   = gre_hdr_size;
 			data 	 = payload;
 			data_len = payload_len;
@@ -776,9 +825,8 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 
 			} break;
 		default:
-			// not handled protocol
-			Free_Node(Node);
-			pcap_dev->proc_stat.unknown++;
+			// other IP protocols are treated as stream such as ESP etc.
+			Push_Node(NodeList, Node);
 			break;
 	}
 
@@ -786,7 +834,7 @@ pkt->vlans[pkt->vlan_count].pcp = (p[0] >> 5) & 7;
 	if ( defragmented ) {
 		free(defragmented);
 		defragmented = NULL;
-		dbg_printf("Defragmented buffer freed for proto %u", proto);	
+		dbg_printf("Defragmented buffer freed for proto %u", IPproto);	
 	}
 
 
