@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2020, Peter Haag
+ *  Copyright (c) 2009-2021, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -141,6 +141,7 @@ static void usage(char *name) {
 					"-I Ident\tset the ident string for stat file. (default 'none')\n"
 					"-H Add port histogram data to flow file.(default 'no')\n"
 					"-n Ident,IP,logdir\tAdd this flow source - multiple streams\n" 
+					"-N sourceFile\tAdd flows from sourceFile\n" 
 					"-P pidfile\tset the PID file\n"
 					"-R IP[/port]\tRepeat incoming packets to IP address/port. Max 8 repeaters\n"
 					"-x process\tlaunch process after a new file becomes available\n"
@@ -716,7 +717,7 @@ char	*pcap_file = NULL;
 	FlowSource		= NULL;
 	extension_tags	= DefaultExtensions;
 
-	while ((c = getopt(argc, argv, "46ewhEVI:DB:b:f:jl:n:p:J:P:R:S:T:t:x:ru:g:zZ")) != EOF) {
+	while ((c = getopt(argc, argv, "46ewhEVI:DB:b:f:jl:N:n:p:J:P:R:S:T:t:x:ru:g:yzZ")) != EOF) {
 		switch (c) {
 			case 'h':
 				usage(argv[0]);
@@ -761,6 +762,10 @@ char	*pcap_file = NULL;
 				if ( AddFlowSource(&FlowSource, optarg) != 1 ) 
 					exit(255);
 				break;
+			case 'N':
+				if ( AddFlowSourceFromFile(&FlowSource, optarg) )
+					exit(255);
+				break;
 			case 'w':
 				synctime = 1;
 				break;
@@ -846,13 +851,21 @@ char	*pcap_file = NULL;
 			case 'l':
 				datadir = optarg;
 				if ( strlen(datadir) > MAXPATHLEN ) {
-					fprintf(stderr, "ERROR: Path too long!\n");
+					LogError("ERROR: Path too long!");
+					exit(255);
+				}
+				if ( stat(datadir, &fstat) < 0 ) {
+					LogError("stat() failed on %s: %s", datadir, strerror(errno));
 					exit(255);
 				}
-				stat(datadir, &fstat);
 				if ( !(fstat.st_mode & S_IFDIR) ) {
-					fprintf(stderr, "No such directory: %s\n", datadir);
-					break;
+					LogError("No such directory: %s", datadir);
+					exit(255);
+				}
+				datadir = realpath(datadir, NULL);
+				if ( !datadir ) {
+					LogError("Can not resolve realpath of %s", optarg);
+					exit(255);
 				}
 				break;
 			case 'T': {
