@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2009-2020, Peter Haag
+ *  Copyright (c) 2009-2021, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -119,6 +119,7 @@ static uint16_t sflow_extensions[] = {
 	EX_MULIPLE, 
 	EX_VLAN, 
 	EX_MAC_1, 
+	EX_MPLS,
 	EX_RECEIVED,
 	0 			// final token
 };
@@ -202,6 +203,12 @@ int i, id;
 void Process_sflow(void *in_buff, ssize_t in_buff_cnt, FlowSource_t *fs) {
 SFSample 	sample;
 int 		exceptionVal;
+#ifdef DEVEL
+static int pkg_num = 0;
+
+	pkg_num++;
+	printf("Process_sflow: Next packet: %i\n", pkg_num);
+#endif
 
 	memset(&sample, 0, sizeof(sample));
 	sample.rawSample = in_buff;
@@ -472,7 +479,8 @@ uint64_t _bytes, _packets, _t;	// tmp buffers
 	_t							  = 1000LL * now.tv_sec + common_record->msec_first;	// tmp buff for first_seen
 
 	common_record->fwd_status	  = 0;
-	common_record->reserved	  	  = 0;
+	common_record->biFlowDir	  = 0;
+	common_record->flowEndReason  = 0;
 	common_record->nfversion	  = 0x80 | sample->datagramVersion;
 	common_record->tcp_flags	  = sample->dcd_tcpFlags;
 	common_record->prot			  = sample->dcd_ipProtocol;
@@ -554,6 +562,16 @@ uint64_t _bytes, _packets, _t;	// tmp buffers
 				tpl->out_dst_mac = Get_val48((void *)&sample->eth_dst);
 				next_data = (void *)tpl->data;
 			} break;
+			case EX_MPLS: {
+				tpl_ext_22_t *tpl = (tpl_ext_22_t *)next_data;
+				for (int i=0; i<10; i++) {
+					if (i<sample->mpls_num_labels)
+						tpl->mpls_label[i] = sample->mpls_label[i];
+					else
+						tpl->mpls_label[i] = 0;
+				}
+				next_data = (void *)tpl->data;
+			} break;
 			case EX_NEXT_HOP_v4:	 {	// next hop IPv4 router address
 				tpl_ext_9_t *tpl = (tpl_ext_9_t *)next_data;
 				if ( sample->nextHop.type == SFLADDRESSTYPE_IP_V4 ) {
