@@ -96,7 +96,7 @@ void xsleep(long sec);
 
 void EndLog(void);
 
-int InitLog(int use_syslog, char *name, char *facility, int verbose_log);
+int InitLog(int want_syslog, char *name, char *facility, int verbose_log);
 
 void LogError(char *format, ...);
 
