@@ -1,19 +1,19 @@
-/* A Bison parser, made by GNU Bison 2.7.  */
+/* A Bison parser, made by GNU Bison 3.0.4.  */
 
 /* Bison interface for Yacc-like parsers in C
-   
-      Copyright (C) 1984, 1989-1990, 2000-2012 Free Software Foundation, Inc.
-   
+
+   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
+
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
@@ -26,13 +26,13 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-   
+
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
 #ifndef YY_FF2_FFILTER_FFILTER_GRAM_H_INCLUDED
 # define YY_FF2_FFILTER_FFILTER_GRAM_H_INCLUDED
-/* Enabling traces.  */
+/* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
 #endif
@@ -40,31 +40,30 @@
 extern int ff2_debug;
 #endif
 
-/* Tokens.  */
+/* Token type.  */
 #ifndef YYTOKENTYPE
 # define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     AND = 258,
-     OR = 259,
-     NOT = 260,
-     ANY = 261,
-     EXIST = 262,
-     EQ = 263,
-     LT = 264,
-     GT = 265,
-     ISSET = 266,
-     IN = 267,
-     IDENT = 268,
-     STRING = 269,
-     QUOTED = 270,
-     DIR = 271,
-     DIR_2 = 272,
-     PAIR_AND = 273,
-     PAIR_OR = 274,
-     BAD_TOKEN = 275
-   };
+  enum yytokentype
+  {
+    AND = 258,
+    OR = 259,
+    NOT = 260,
+    ANY = 261,
+    EXIST = 262,
+    EQ = 263,
+    LT = 264,
+    GT = 265,
+    ISSET = 266,
+    IN = 267,
+    IDENT = 268,
+    STRING = 269,
+    QUOTED = 270,
+    DIR = 271,
+    DIR_2 = 272,
+    PAIR_AND = 273,
+    PAIR_OR = 274,
+    BAD_TOKEN = 275
+  };
 #endif
 /* Tokens.  */
 #define AND 258
@@ -86,41 +85,28 @@ extern int ff2_debug;
 #define PAIR_OR 274
 #define BAD_TOKEN 275
 
-
-
+/* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE
+
+union YYSTYPE
 {
-/* Line 2058 of yacc.c  */
-#line 47 "ffilter/ffilter_gram.y"
+#line 47 "ffilter/ffilter_gram.y" /* yacc.c:1909  */
 
 	uint64_t	t_uint;
 	double		t_double;
 	char 		string[FF_MAX_STRING];
 	void		*node;
 
+#line 101 "ffilter/ffilter_gram.h" /* yacc.c:1909  */
+};
 
-/* Line 2058 of yacc.c  */
-#line 105 "ffilter/ffilter_gram.h"
-} YYSTYPE;
+typedef union YYSTYPE YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
-# define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 #endif
 
 
-#ifdef YYPARSE_PARAM
-#if defined __STDC__ || defined __cplusplus
-int ff2_parse (void *YYPARSE_PARAM);
-#else
-int ff2_parse ();
-#endif
-#else /* ! YYPARSE_PARAM */
-#if defined __STDC__ || defined __cplusplus
+
 int ff2_parse (yyscan_t scanner, ff_t *filter);
-#else
-int ff2_parse ();
-#endif
-#endif /* ! YYPARSE_PARAM */
 
 #endif /* !YY_FF2_FFILTER_FFILTER_GRAM_H_INCLUDED  */
