@@ -321,7 +321,7 @@ void lnf_filedlist_free(lnf_fieldlist_t *list) {
 
 	node = list;
 
-	while (node == NULL) {
+	while (node != NULL) {
 		tmp_node = node;
 		node = node->next;
 		free(tmp_node);
@@ -382,7 +382,6 @@ int lnf_mem_fadd(lnf_mem_t *lnf_mem, int field, int flags, int numbits, int numb
 
 	fld.sort_flag = flags & LNF_SORT_FLAGS;
 		
-
 	/* select aggregation func for item */
 	fld.aggr_func = lnf_mem_aggr_EMPTY;
 	switch (lnf_fld_type(fld.field)) {
@@ -450,7 +449,7 @@ int lnf_mem_fadd(lnf_mem_t *lnf_mem, int field, int flags, int numbits, int numb
 		}
 	}
 
-	/* if the filed is calculated field with dependencies add filds that the firld is depenedend on */
+	/* if the filed is calculated field with dependencies add fields that the field is depenedend on */
 	calcnum = 0;
 	while ( __lnf_fld_calc_dep(field, calcnum) != LNF_FLD_ZERO_) {
 
@@ -760,9 +759,9 @@ int lnf_mem_write(lnf_mem_t *lnf_mem, lnf_rec_t *rec) {
 		if (id == NULL) {
 			return LNF_ERR_OTHER;
 		}
-		lnf_mem->thread_status[*id] = LNF_TH_WRITE;
 	}
 
+	lnf_mem->thread_status[*id] = LNF_TH_WRITE;
 
 	if (lnf_mem->list_mode) {
 		if (hash_table_insert_list(&lnf_mem->hash_table[*id], keybuf, valbuf) == NULL) {
@@ -1084,8 +1083,23 @@ void lnf_mem_read_reset(lnf_mem_t *lnf_mem) {
 
 }
 
+/* make lnf_mem object empty */
+void lnf_mem_clean(lnf_mem_t *lnf_mem) {
+
+	if (lnf_mem == NULL) {
+		return;
+	}
+
+	if (lnf_mem->thread_status[0] != LNF_TH_EMPTY) {
+		hash_table_clean(&lnf_mem->hash_table[0]);
+		lnf_mem->thread_status[0] = LNF_TH_EMPTY;
+	}
+}
+
 void lnf_mem_free(lnf_mem_t *lnf_mem) {
 
+	void *tmp;
+
 	if (lnf_mem == NULL) {
 		return;
 	}
@@ -1103,9 +1117,16 @@ void lnf_mem_free(lnf_mem_t *lnf_mem) {
 	}
 
 #ifdef LNF_THREADS
+	if ((tmp = pthread_getspecific(lnf_mem->thread_id_key)) != NULL) {
+		free(tmp);
+	}
 	pthread_key_delete(lnf_mem->thread_id_key);
+#else
+	free((void *)lnf_mem->thread_id_key);
 #endif
 
+	lnf_rec_free(lnf_mem->lnf_rec);
+
 	free(lnf_mem);
 
 }
