@@ -1,6 +1,5 @@
 /*
- *  Copyright (c) 2014, Peter Haag
- *  Copyright (c) 2011, Peter Haag
+ *  Copyright (c) 2009-2020, Peter Haag
  *  Copyright (c) 2004-2008, SWITCH - Teleinformatikdienste fuer Lehre und Forschung
  *  All rights reserved.
  *  
@@ -28,12 +27,6 @@
  *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  *  POSSIBILITY OF SUCH DAMAGE.
  *  
- *  $Author: haag $
- *
- *  $Id: nffile.c 41 2009-12-31 14:46:28Z haag $
- *
- *  $LastChangedRevision: 41 $
- *	
  */
 
 #include "config.h"
@@ -43,6 +36,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <sys/uio.h>
+#include <time.h>
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
@@ -52,46 +46,43 @@
 #include <errno.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <bzlib.h>
 
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
 
-#include "minilzo.h"
-#include "nf_common.h"
-#include "nffile.h"
 #include "util.h"
+#include "nfdump.h"
+#include "minilzo.h"
+#include "lz4.h"
+#include "flist.h"
+#include "nffile.h"
+#include "nffileV2.h"
 
 /* global vars */
 
-#ifdef LNF_THREADS
-static __thread int open_files = 0;
-#else
-static int open_files = 0;
-#endif
-
-
 // required for idet filter in nftree.c
 char 	*CurrentIdent;
 
 
-#define READ_FILE	1
-#define WRITE_FILE	1
-
 // LZO params
-#define LZO_BUFFSIZE  ((BUFFSIZE + BUFFSIZE / 16 + 64 + 3) + sizeof(data_block_header_t))
 #define HEAP_ALLOC(var,size) \
     lzo_align_t __LZO_MMODEL var [ ((size) + (sizeof(lzo_align_t) - 1)) / sizeof(lzo_align_t) ]
 
 static HEAP_ALLOC(wrkmem,LZO1X_1_MEM_COMPRESS);
-static void *lzo_buff;
 static int lzo_initialized = 0;
-
-#define ERR_SIZE 256
-static char	error_string[ERR_SIZE];
+static int lz4_initialized = 0;
+static int bz2_initialized = 0;
 
 static int LZO_initialize(void);
 
+static int LZ4_initialize(void);
+
+static int BZ2_initialize(void);
+
+static void BZ2_prep_stream (bz_stream*);
+
 static int OpenRaw(char *filename, stat_record_t *stat_record, int *compressed);
 
 extern char *nf_error;
@@ -142,22 +133,257 @@ void SumStatRecords(stat_record_t *s1, stat_record_t *s2) {
 static int LZO_initialize(void) {
 
 	if (lzo_init() != LZO_E_OK) {
-			// this usually indicates a compiler bug - try recompiling 
-			// without optimizations, and enable `-DLZO_DEBUG' for diagnostics
-			LogError("Compression lzo_init() failed.\n");
-			return 0;
-	} 
-	lzo_buff = malloc(BUFFSIZE+ sizeof(data_block_header_t));
-	if ( !lzo_buff ) {
-		LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+		// this usually indicates a compiler bug - try recompiling 
+		// without optimizations, and enable `-DLZO_DEBUG' for diagnostics
+		LogError("Compression lzo_init() failed.\n");
 		return 0;
-	}
+	} 
 	lzo_initialized = 1;
 
 	return 1;
 
 } // End of LZO_initialize
 
+static int LZ4_initialize (void) {
+
+	int lz4_buff_size = LZ4_compressBound(BUFFSIZE + sizeof (data_block_header_t));
+	if ( lz4_buff_size > (2 * BUFFSIZE) ) {
+		LogError ("LZ4_compressBound() error in %s line %d: Buffer too small\n", __FILE__, __LINE__);
+		return 0;
+	}
+	lz4_initialized = 1;
+
+	return 1;
+
+} // End of LZ4_initialize
+
+static int BZ2_initialize (void) {
+
+	bz2_initialized = 1;
+
+	return 1;
+
+} // End of BZ2_initialize
+
+static void BZ2_prep_stream (bz_stream* bs)
+{
+   bs->bzalloc = NULL;
+   bs->bzfree = NULL;
+   bs->opaque = NULL;
+} // End of BZ2_prep_stream
+
+static int Compress_Block_LZO(nffile_t *nffile) {
+unsigned char __LZO_MMODEL *in;
+unsigned char __LZO_MMODEL *out;
+lzo_uint in_len;
+lzo_uint out_len;
+int r;
+
+	in  = (unsigned char __LZO_MMODEL *)(nffile->buff_pool[0] + sizeof(data_block_header_t));	
+	out = (unsigned char __LZO_MMODEL *)(nffile->buff_pool[1] + sizeof(data_block_header_t));	
+	in_len = nffile->block_header->size;
+	r = lzo1x_1_compress(in,in_len,out,&out_len,wrkmem);
+
+	if (r != LZO_E_OK) {
+		LogError("Compress_Block_LZO() error compression failed in %s line %d: LZ4 : %d\n", __FILE__, __LINE__, r);
+		return -1;
+	}
+	
+	// copy header
+	memcpy(nffile->buff_pool[1], nffile->buff_pool[0], sizeof(data_block_header_t));
+	((data_block_header_t *)nffile->buff_pool[1])->size = out_len;
+
+	// swap buffers
+	void *_tmp = nffile->buff_pool[1];
+	nffile->buff_pool[1] = nffile->buff_pool[0];
+	nffile->buff_pool[0] = _tmp;
+
+	nffile->block_header = nffile->buff_pool[0];
+
+	return 1;
+
+} // End of Compress_Block_LZO
+
+static int Uncompress_Block_LZO(nffile_t *nffile) {
+unsigned char __LZO_MMODEL *in;
+unsigned char __LZO_MMODEL *out;
+lzo_uint in_len;
+lzo_uint out_len;
+int r;
+
+	in  = (unsigned char __LZO_MMODEL *)(nffile->buff_pool[0] + sizeof(data_block_header_t));	
+	out = (unsigned char __LZO_MMODEL *)(nffile->buff_pool[1] + sizeof(data_block_header_t));	
+	in_len  = nffile->block_header->size;
+	out_len = nffile->buff_size;
+
+	if ( in_len == 0 ) {
+		LogError("Uncompress_Block_LZO() header length error in %s line %d\n", __FILE__, __LINE__);
+   		return -1;
+	}
+	r = lzo1x_decompress_safe(in,in_len,out,&out_len,NULL);
+	if (r != LZO_E_OK ) {
+  		/* this should NEVER happen */
+		LogError("Uncompress_Block_LZO() error decompression failed in %s line %d: LZO error: %d\n", __FILE__, __LINE__, r);
+   		return -1;
+	}
+
+	// copy header
+	memcpy(nffile->buff_pool[1], nffile->buff_pool[0], sizeof(data_block_header_t));
+	((data_block_header_t *)nffile->buff_pool[1])->size = out_len;
+
+	// swap buffers
+	void *_tmp = nffile->buff_pool[1];
+	nffile->buff_pool[1] = nffile->buff_pool[0];
+	nffile->buff_pool[0] = _tmp;
+
+	nffile->block_header = nffile->buff_pool[0];
+	nffile->buff_ptr 	 = nffile->buff_pool[0] + sizeof(data_block_header_t);
+
+	return 1;
+
+} // End of Uncompress_Block_LZO
+
+static int Compress_Block_LZ4(nffile_t *nffile) {
+
+	const char *in  = (const char *)(nffile->buff_pool[0] + sizeof(data_block_header_t));
+	char *out 		= (char *)(nffile->buff_pool[1] + sizeof(data_block_header_t));
+	int in_len 		= nffile->block_header->size;
+
+	int out_len = LZ4_compress_default(in, out, in_len, nffile->buff_size);
+	if (out_len == 0 ) {
+		LogError("Compress_Block_LZ4() error compression aborted in %s line %d: LZ4 : buffer too small\n", __FILE__, __LINE__);
+   		return -1;
+   	}
+   	if (out_len < 0 ) {
+		LogError("Compress_Block_LZ4() error compression failed in %s line %d: LZ4 : %d\n", __FILE__, __LINE__, out_len);
+   		return -1;
+   	}
+
+	// copy header
+	memcpy(nffile->buff_pool[1], nffile->buff_pool[0], sizeof(data_block_header_t));
+	((data_block_header_t *)nffile->buff_pool[1])->size = out_len;
+
+	// swap buffers
+	void *_tmp = nffile->buff_pool[1];
+	nffile->buff_pool[1] = nffile->buff_pool[0];
+	nffile->buff_pool[0] = _tmp;
+
+	nffile->block_header = nffile->buff_pool[0];
+
+	return 1;
+
+} // End of Compress_Block_LZ4
+
+static int Uncompress_Block_LZ4(nffile_t *nffile) {
+
+	const char *in  = (const char *)(nffile->buff_pool[0] + sizeof(data_block_header_t));
+	char *out 		= (char *)(nffile->buff_pool[1] + sizeof(data_block_header_t));
+	int in_len 		= nffile->block_header->size;
+
+	int out_len = LZ4_decompress_safe(in, out, in_len, nffile->buff_size);
+	if (out_len == 0 ) {
+		LogError("LZ4_decompress_safe() error compression aborted in %s line %d: LZ4 : buffer too small\n", __FILE__, __LINE__);
+   		return -1;
+   	}
+   	if (out_len < 0 ) {
+		LogError("LZ4_decompress_safe() error compression failed in %s line %d: LZ4 : %d\n", __FILE__, __LINE__, out_len);
+   		return -1;
+   	}
+
+	// copy header
+	memcpy(nffile->buff_pool[1], nffile->buff_pool[0], sizeof(data_block_header_t));
+	((data_block_header_t *)nffile->buff_pool[1])->size = out_len;
+
+	// swap buffers
+	void *_tmp = nffile->buff_pool[1];
+	nffile->buff_pool[1] = nffile->buff_pool[0];
+	nffile->buff_pool[0] = _tmp;
+
+	nffile->block_header = nffile->buff_pool[0];
+	nffile->buff_ptr 	 = nffile->buff_pool[0] + sizeof(data_block_header_t);
+
+	return 1;
+
+} // End of Uncompress_Block_LZ4
+
+static int Compress_Block_BZ2(nffile_t *nffile) {
+bz_stream bs;
+
+	BZ2_prep_stream (&bs);
+	BZ2_bzCompressInit (&bs, 9, 0, 0);
+
+	bs.next_in   = (char*)(nffile->buff_pool[0] + sizeof(data_block_header_t));
+	bs.next_out  = (char*)(nffile->buff_pool[1] + sizeof(data_block_header_t));
+	bs.avail_in  = nffile->block_header->size;
+	bs.avail_out = nffile->buff_size;
+ 
+	for (;;) {
+		int r = BZ2_bzCompress (&bs, BZ_FINISH);
+		if (r == BZ_FINISH_OK) continue;
+		if (r != BZ_STREAM_END) {
+			LogError("Compress_Block_BZ2() error compression failed in %s line %d: LZ4 : %d\n", __FILE__, __LINE__, r);
+			return -1;
+		}
+		break;
+	}
+
+ 	// copy header
+	memcpy(nffile->buff_pool[1], nffile->buff_pool[0], sizeof(data_block_header_t));
+	((data_block_header_t *)nffile->buff_pool[1])->size = bs.total_out_lo32;
+
+	// swap buffers
+	void *_tmp = nffile->buff_pool[1];
+	nffile->buff_pool[1] = nffile->buff_pool[0];
+	nffile->buff_pool[0] = _tmp;
+
+	nffile->block_header = nffile->buff_pool[0];
+
+	BZ2_bzCompressEnd (&bs);
+	
+	return 1;
+
+} // End of Compress_Block_BZ2
+
+static int Uncompress_Block_BZ2(nffile_t *nffile) {
+bz_stream bs;
+
+	BZ2_prep_stream (&bs);
+	BZ2_bzDecompressInit (&bs, 0, 0);
+
+	bs.next_in   = (char*)(nffile->buff_pool[0] + sizeof(data_block_header_t));
+	bs.next_out  = (char*)(nffile->buff_pool[1] + sizeof(data_block_header_t));
+	bs.avail_in  = nffile->block_header->size;
+	bs.avail_out = nffile->buff_size;
+ 
+	for (;;) {
+		int r = BZ2_bzDecompress (&bs);
+		if (r == BZ_OK) {
+			continue;
+		} else if (r != BZ_STREAM_END) {
+			BZ2_bzDecompressEnd (&bs);
+			return NF_CORRUPT;
+		} else {
+			break;
+		}
+	}
+
+ 	// copy header
+	memcpy(nffile->buff_pool[1], nffile->buff_pool[0], sizeof(data_block_header_t));
+	((data_block_header_t *)nffile->buff_pool[1])->size = bs.total_out_lo32;
+
+	// swap buffers
+	void *_tmp = nffile->buff_pool[1];
+	nffile->buff_pool[1] = nffile->buff_pool[0];
+	nffile->buff_pool[0] = _tmp;
+
+	nffile->block_header = nffile->buff_pool[0];
+	nffile->buff_ptr 	 = nffile->buff_pool[0] + sizeof(data_block_header_t);
+
+	BZ2_bzDecompressEnd (&bs);
+	
+	return 1;
+
+} // End of Uncompress_Block_BZ2
 
 nffile_t *OpenFile(char *filename, nffile_t *nffile){
 struct stat stat_buf;
@@ -238,14 +464,29 @@ int ret, allocated;
 
 	CurrentIdent		= nffile->file_header->ident;
 
-	if ( FILE_IS_COMPRESSED(nffile) && !lzo_initialized && !LZO_initialize() ) {
-		if ( allocated ) {
-			DisposeFile(nffile);
-			return NULL;
-		}
-    }
-
-	open_files++;
+	int compression = FILE_COMPRESSION(nffile);
+	switch (compression) {
+		case NOT_COMPRESSED:
+			break;
+		case LZO_COMPRESSED: 
+			if ( !lzo_initialized && !LZO_initialize() && allocated ) {
+				DisposeFile(nffile);
+				return NULL;
+			}
+			break;
+		case LZ4_COMPRESSED: 
+			if ( !lz4_initialized && !LZ4_initialize() && allocated ) {
+				DisposeFile(nffile);
+				return NULL;
+			}
+			break;
+		case BZ2_COMPRESSED: 
+			if ( !bz2_initialized && !BZ2_initialize() && allocated ) {
+				DisposeFile(nffile);
+				return NULL;
+			}
+			break;
+	}
 
 	return nffile;
 
@@ -260,15 +501,6 @@ void CloseFile(nffile_t *nffile){
 	if ( nffile->fd )
 		close(nffile->fd);
 
-	open_files--;
-
-	if (open_files == 0 && lzo_initialized && lzo_buff) {
-		free(lzo_buff);
-		lzo_buff = NULL;
-		lzo_initialized = 0;
-	}
-
-
 } // End of CloseFile
 
 int ChangeIdent(char *filename, char *Ident) {
@@ -366,6 +598,7 @@ void PrintStat(stat_record_t *s) {
 
 static nffile_t *NewFile(void) {
 nffile_t *nffile;
+int i;
 
 	// Create struct
 	nffile = calloc(1, sizeof(nffile_t));
@@ -375,7 +608,6 @@ nffile_t *nffile;
 	}
 	nffile->buff_ptr = NULL;
 	nffile->fd	 	= 0;
-	nffile->catalog = NULL;
 
 	// Init file header
 	nffile->file_header = calloc(1, sizeof(file_header_t));
@@ -394,50 +626,86 @@ nffile_t *nffile;
 		return NULL;
 	}
 
-/*
-	XXX catalogs not yet implemented
-	nffile->catalog = calloc(1, sizeof(catalog_t));
-	if ( !nffile->catalog ) {
-		LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
-		return NULL;
-	}
-	nffile->catalog->NumRecords = 0;
-	nffile->catalog->size 		= sizeof(catalog_t) - sizeof(data_block_header_t);
-	nffile->catalog->id 		= CATALOG_BLOCK;
-	nffile->catalog->pad 		= 0;
-	nffile->catalog->reserved 	= 0;
-*/
 	// init data buffer
-	nffile->block_header = malloc(BUFFSIZE + sizeof(data_block_header_t));
-	if ( !nffile->block_header ) {
-		LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
-		return NULL;
+	nffile->buff_size = 2 * BUFFSIZE;
+	for (i=0; i<NUM_BUFFS; i++ ) {
+		// allocate twice of BUFFSIZE initially - should be ok, otherwise expand
+		nffile->buff_pool[i] = malloc(nffile->buff_size);
+		if ( !nffile->buff_pool[i] ) {
+			LogError("malloc() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
+			return NULL;
+		}
 	}
+
+	nffile->block_header 			 = nffile->buff_pool[0];
 	nffile->block_header->size 		 = 0;
 	nffile->block_header->NumRecords = 0;
 	nffile->block_header->id		 = DATA_BLOCK_TYPE_2;
 	nffile->block_header->flags		 = 0;
 
 	nffile->buff_ptr = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t));
-
-	open_files++;
 	
 	return nffile;
 
 } // End of NewFile
 
 nffile_t *DisposeFile(nffile_t *nffile) {
+int i;
+
 	free(nffile->file_header);
 	free(nffile->stat_record);
-	if (nffile->block_header) 
-		free(nffile->block_header);
-	free(nffile);
+
+	for (i=0; i<NUM_BUFFS; i++ ) {
+		free(nffile->buff_pool[i]);
+	}
+
 	return NULL;
 } // End of DisposeFile
 
-nffile_t *OpenNewFile(char *filename, nffile_t *nffile, int compressed, int anonymized, char *ident) {
+nffile_t *OpenNewFile(char *filename, nffile_t *nffile, int compress, int anonymized, char *ident) {
 size_t			len;
-int 			flags;
+int 			fd, flags;
+
+	switch (compress) {
+		case NOT_COMPRESSED:
+			flags = FLAG_NOT_COMPRESSED;
+			break;
+		case LZO_COMPRESSED:
+			flags = FLAG_LZO_COMPRESSED;
+			if ( !lzo_initialized && !LZO_initialize() ) {
+				LogError("Failed to initialize LZO compression");
+				return NULL;
+			}
+			break;
+		case LZ4_COMPRESSED:
+			flags = FLAG_LZ4_COMPRESSED;
+			if ( !lz4_initialized && !LZ4_initialize() ) {
+				LogError("Failed to initialize LZ4 compression");
+				return NULL;
+			}
+			break;
+		case BZ2_COMPRESSED:
+			flags = FLAG_BZ2_COMPRESSED;
+			if ( !bz2_initialized && !BZ2_initialize() ) {
+				LogError("Failed to initialize BZ2 compression");
+				return NULL;
+			}
+			break;
+		default:
+			LogError("Unknown compression ID: %i\n", compress);
+			return NULL;
+	}
+
+	fd = 0;
+	if ( strcmp(filename, "-") == 0 ) { // output to stdout
+		fd = STDOUT_FILENO;
+	} else {
+		fd = open(filename, O_CREAT | O_RDWR | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH );
+		if ( fd < 0 ) {
+			LogError("Failed to open file %s: '%s'" , filename, strerror(errno));
+			return NULL;
+		}
+	}
 
 	// Allocate new struct if not given
 	if ( nffile == NULL ) {
@@ -447,30 +715,13 @@ int 			flags;
 		}
 	}
 
-	flags = compressed ? FLAG_COMPRESSED : 0;
+	nffile->fd = fd;
+
 	if ( anonymized ) 
 		SetFlag(flags, FLAG_ANONYMIZED);
 
 	nffile->file_header->flags 	   = flags;
 
-	if ( strcmp(filename, "-") == 0 ) { // output to stdout
-		nffile->fd = STDOUT_FILENO;
-	} else {
-		nffile->fd = open(filename, O_CREAT | O_RDWR | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH );
-		if ( nffile->fd < 0 ) {
-			LogError("Failed to open file %s: '%s'" , filename, strerror(errno));
-			return NULL;
-		}
-	}
-
-/*
-	XXX catalogs not yet implemented
-	if ( nffile->catalog && nffile->catalog->NumRecords ) {
-		memset((void *)nffile->catalog->entries, 0, nffile->catalog->NumRecords * sizeof(struct catalog_entry_s));
-		nffile->catalog->NumRecords = 0;
-		nffile->catalog->size		= 0;
-	} 
-*/
 	if ( nffile->stat_record ) {
 		memset((void *)nffile->stat_record, 0, sizeof(stat_record_t));
 		nffile->stat_record->first_seen = 0x7fffffff;
@@ -478,24 +729,16 @@ int 			flags;
 	}
 
 	if ( ident ) {
-		strncpy(nffile->file_header->ident, ident, IDENTLEN);
+		strncpy(nffile->file_header->ident, ident, IDENTLEN-1);
 		nffile->file_header->ident[IDENTLEN - 1] = 0;
 	} 
 
-
-	if ( TestFlag(flags, FLAG_COMPRESSED) ) {
-		if ( !lzo_initialized && !LZO_initialize() ) {
-			LogError("Failed to initialize compression");
-			close(nffile->fd);
-			return NULL;
-		}
-    	}
-
 	nffile->file_header->NumBlocks = 0;
 	len = sizeof(file_header_t);
 	if ( write(nffile->fd, (void *)nffile->file_header, len) < len ) {
 		LogError("write() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
 		close(nffile->fd);
+		nffile->fd = 0;
 		return NULL;
 	}
 
@@ -504,18 +747,10 @@ int 			flags;
 	if ( write(nffile->fd, (void *)nffile->stat_record, len) < len ) {
 		LogError("write() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
 		close(nffile->fd);
+		nffile->fd = 0;
 		return NULL;
 	}
 
-/* skip writing catalog in this test version
-	XXX catalogs not yet implemented
-	if ( WriteExtraBlock(nffile, (data_block_header_t *)nffile->catalog) < 0 ) {
-		LogError("write() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
-		close(nffile->fd);
-		return NULL;
-	}
-*/
-
 	return nffile;
 
 } /* End of OpenNewFile */
@@ -551,15 +786,35 @@ nffile_t		*nffile;
 	nffile->block_header->flags		 = 0;
 	nffile->buff_ptr = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t));
 
-	// initialize output  lzo buffer
-	if ( FILE_IS_COMPRESSED(nffile) ) {
-		if ( !lzo_initialized && !LZO_initialize() ) {
-			LogError("Failed to initialize compression");
-			close(nffile->fd);
-			DisposeFile(nffile);
-			return NULL;
-		}
-    }
+	int compression = FILE_COMPRESSION(nffile);
+	switch (compression) {
+		case NOT_COMPRESSED:
+			break;
+		case LZO_COMPRESSED: 
+			if ( !lzo_initialized && !LZO_initialize() ) {
+				LogError("Failed to initialize LZO compression");
+				close(nffile->fd);
+				DisposeFile(nffile);
+				return NULL;
+			}
+			break;
+		case LZ4_COMPRESSED: 
+			if ( !lz4_initialized && !LZ4_initialize() ) {
+				LogError("Failed to initialize LZ4 compression");
+				close(nffile->fd);
+				DisposeFile(nffile);
+				return NULL;
+			}
+			break;
+		case BZ2_COMPRESSED: 
+			if ( !bz2_initialized && !BZ2_initialize() ) {
+				LogError("Failed to initialize BZ2 compression");
+				close(nffile->fd);
+				DisposeFile(nffile);
+				return NULL;
+			}
+			break;
+	}
 
 	return nffile;
 
@@ -704,8 +959,16 @@ int fd, ret;
 		close(fd);
 		return -1;
 	}
-	
-	*compressed = file_header.flags & FLAG_COMPRESSED ? 1 : 0;
+
+	if ( file_header.flags & FLAG_LZO_COMPRESSED ) 
+		*compressed = FLAG_LZO_COMPRESSED;
+	else if ( file_header.flags & FLAG_LZ4_COMPRESSED )
+		*compressed = FLAG_LZ4_COMPRESSED;
+	else if ( file_header.flags & FLAG_BZ2_COMPRESSED )
+		*compressed = FLAG_BZ2_COMPRESSED;
+	else
+		*compressed = 0;
+
 	return fd;
 
 } // End of OpenRaw
@@ -734,11 +997,12 @@ int CloseUpdateFile(nffile_t *nffile, char *ident) {
 	}
 
 	if ( ident ) {
-		strncpy(nffile->file_header->ident, ident, IDENTLEN);
+		strncpy(nffile->file_header->ident, ident, IDENTLEN-1);
 	} else {
 		if ( strlen(nffile->file_header->ident) == 0 ) 
-		strncpy(nffile->file_header->ident, IDENTNONE, IDENTLEN);
+		strncpy(nffile->file_header->ident, IDENTNONE, IDENTLEN-1);
 	}
+	nffile->file_header->ident[IDENTLEN-1] = 0;
 
 	if ( write(nffile->fd, (void *)nffile->file_header, sizeof(file_header_t)) <= 0 ) {
 		LogError("write() error in %s line %d: %s\n", __FILE__, __LINE__, strerror(errno) );
@@ -759,7 +1023,8 @@ int CloseUpdateFile(nffile_t *nffile, char *ident) {
 
 int ReadBlock(nffile_t *nffile) {
 ssize_t ret, read_bytes, buff_bytes, request_size;
-void 	*read_ptr, *buff;
+void 	*read_ptr;
+uint32_t compression;
 
 	ret = read(nffile->fd, nffile->block_header, sizeof(data_block_header_t));
 	if ( ret == 0 )		// EOF
@@ -779,30 +1044,37 @@ void 	*read_ptr, *buff;
 	read_bytes = ret;
 
 	// Check for sane buffer size
-	if ( nffile->block_header->size > BUFFSIZE ) {
+	if ( nffile->block_header->size > BUFFSIZE ||
+	     nffile->block_header->size == 0 || nffile->block_header->NumRecords == 0) {
 		// this is most likely a corrupt file
-		LogError("Corrupt data file: Requested buffer size %u exceeds max. buffer size.\n", nffile->block_header->size);
+		LogError("Corrupt data file: Requested buffer size %u exceeds max. buffer size", nffile->block_header->size);
 		return NF_CORRUPT;
 	}
 
-	buff = FILE_IS_COMPRESSED(nffile) ? lzo_buff : nffile->buff_ptr;
+	// check block compression - defaults to file compression setting
 
-	ret = read(nffile->fd, buff, nffile->block_header->size);
+	compression = FILE_COMPRESSION(nffile);
+	ret = read(nffile->fd, nffile->buff_ptr, nffile->block_header->size);
 	if ( ret == nffile->block_header->size ) {
-		lzo_uint new_len;
 		// we have the whole record and are done for now
-		if ( FILE_IS_COMPRESSED(nffile) ) {
-			int r;
-   			r = lzo1x_decompress(lzo_buff,nffile->block_header->size,nffile->buff_ptr,&new_len,NULL);
-   			if (r != LZO_E_OK ) {
-       			/* this should NEVER happen */
-				LogError("ReadBlock() error decompression failed in %s line %d: LZO error: %d\n", __FILE__, __LINE__, r);
-       			return NF_CORRUPT;
-   			}
-			nffile->block_header->size = new_len;
-			return read_bytes + new_len;
-		} else
-			return read_bytes + ret;
+		switch (compression) {
+			case NOT_COMPRESSED:
+				break;
+			case LZO_COMPRESSED: 
+				if ( Uncompress_Block_LZO(nffile) < 0 ) 
+					return NF_CORRUPT;
+				break;
+			case LZ4_COMPRESSED: 
+				if ( Uncompress_Block_LZ4(nffile) < 0 ) 
+					return NF_CORRUPT;
+				break;
+			case BZ2_COMPRESSED: 
+				if ( Uncompress_Block_BZ2(nffile) < 0 )
+					return NF_CORRUPT;
+			break;
+		}
+		nffile->buff_ptr = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t));
+		return read_bytes + nffile->block_header->size;
 	} 
 			
 	if ( ret == 0 ) {
@@ -823,7 +1095,7 @@ void 	*read_ptr, *buff;
 	buff_bytes 	 = ret;								// already in buffer
 	request_size = nffile->block_header->size - buff_bytes;	// still to go for this amount of data
 
-	read_ptr 	 = (void *)((pointer_addr_t)buff + buff_bytes);	
+	read_ptr 	 = (void *)((pointer_addr_t)nffile->buff_ptr + buff_bytes);	
 	do {
 		ret = read(nffile->fd, read_ptr, request_size);
 		if ( ret < 0 ) {
@@ -834,7 +1106,7 @@ void 	*read_ptr, *buff;
 
 		if ( ret == 0 ) {
 			//  0: EOF   - not expected
-			LogError(error_string, ERR_SIZE, "Corrupt data file: Unexpected EOF. Short read of data block.\n");
+			LogError("read() corrupt data file: Unexpected EOF in %s line %d: %s\n", __FILE__, __LINE__);
 			return NF_CORRUPT;
 		} 
 		
@@ -843,270 +1115,163 @@ void 	*read_ptr, *buff;
 
 		if ( request_size > 0 ) {
 			// still a short read - continue in read loop
-			read_ptr = (void *)((pointer_addr_t)buff + buff_bytes);
+			read_ptr = (void *)((pointer_addr_t)nffile->buff_ptr + buff_bytes);
 		}
 	} while ( request_size > 0 );
 
-	if ( FILE_IS_COMPRESSED(nffile) ) {
-		int r;
-		lzo_uint new_len;
-   		r = lzo1x_decompress(lzo_buff, nffile->block_header->size, nffile->buff_ptr, &new_len, NULL);
-   		if (r != LZO_E_OK ) {
-       		/* this should NEVER happen */
-			LogError("ReadBlock() error decompression failed in %s line %d: LZO error: %d\n", __FILE__, __LINE__, r);
-       		return NF_CORRUPT;
-   		}
-		nffile->block_header->size = new_len;
-		return read_bytes + new_len;
-
-	} else {
-		// finally - we are done for now
-		return read_bytes + buff_bytes;
+	switch (compression) {
+		case NOT_COMPRESSED:
+			break;
+		case LZO_COMPRESSED: 
+			if ( Uncompress_Block_LZO(nffile) < 0 ) 
+				return NF_CORRUPT;
+			break;
+		case LZ4_COMPRESSED: 
+			if ( Uncompress_Block_LZ4(nffile) < 0 ) 
+				return NF_CORRUPT;
+			break;
+		case BZ2_COMPRESSED: 
+			if ( Uncompress_Block_BZ2(nffile) < 0 )
+				return NF_CORRUPT;
+		break;
 	}
 
-	/* not reached */
+	nffile->buff_ptr = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t));
+	return read_bytes + nffile->block_header->size;
 
 } // End of ReadBlock
 
 int WriteBlock(nffile_t *nffile) {
-data_block_header_t *out_block_header;
-int r, ret;
-unsigned char __LZO_MMODEL *in;
-unsigned char __LZO_MMODEL *out;
-lzo_uint in_len;
-lzo_uint out_len;
+int ret, compression;
 
 	// empty blocks need not to be stored 
 	if ( nffile->block_header->size == 0 )
 		return 1;
 
-	if ( !TestFlag(nffile->file_header->flags, FLAG_COMPRESSED) ) {
-		ret = write(nffile->fd, (void *)nffile->block_header, sizeof(data_block_header_t) + nffile->block_header->size);
-		if ( ret > 0 ) {
-			nffile->block_header->size 		 = 0;
-			nffile->block_header->NumRecords = 0;
-			nffile->buff_ptr = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t) );
-			nffile->file_header->NumBlocks++;
-		}
-		return ret;
-	} 
-
-	out_block_header = (data_block_header_t *)lzo_buff;
-	*out_block_header = *(nffile->block_header);
-
-	in  = (unsigned char __LZO_MMODEL *)((pointer_addr_t)nffile->block_header     + sizeof(data_block_header_t));	
-	out = (unsigned char __LZO_MMODEL *)((pointer_addr_t)out_block_header + sizeof(data_block_header_t));	
-	in_len = nffile->block_header->size;
-	r = lzo1x_1_compress(in,in_len,out,&out_len,wrkmem);
-
-	if (r != LZO_E_OK) {
-		snprintf(error_string, ERR_SIZE,"compression failed: %d" , r);
-		error_string[ERR_SIZE-1] = 0;
-		return -2;
+	compression = FILE_COMPRESSION(nffile);
+	switch (compression) {
+		case NOT_COMPRESSED:
+			break;
+		case LZO_COMPRESSED: 
+			if ( Compress_Block_LZO(nffile) < 0 ) return -1;
+			break;
+		case LZ4_COMPRESSED:
+			if ( Compress_Block_LZ4(nffile) < 0 ) return -1;
+			break;
+		case BZ2_COMPRESSED:
+			if ( Compress_Block_BZ2(nffile) < 0 ) return -1;
+		break;
 	}
 
-	out_block_header->size = out_len;
-	ret = write(nffile->fd, (void *)out_block_header, sizeof(data_block_header_t) + out_block_header->size);
-	if ( ret > 0 ) {
-		nffile->block_header->size 		 = 0;
+	ret = write(nffile->fd, (void *)nffile->block_header, sizeof(data_block_header_t) + nffile->block_header->size);
+	if (ret > 0) {
+		nffile->block_header->size = 0;
 		nffile->block_header->NumRecords = 0;
-		nffile->buff_ptr = (void *)((pointer_addr_t)nffile->block_header + sizeof(data_block_header_t) );
+		nffile->buff_ptr = (void *)((pointer_addr_t) nffile->block_header + sizeof (data_block_header_t));
 		nffile->file_header->NumBlocks++;
 	}
-
+ 	
 	return ret;
 
 } // End of WriteBlock
 
-int WriteExtraBlock(nffile_t *nffile, data_block_header_t *block_header) {
-data_block_header_t *out_block_header;
-int r, ret;
-unsigned char __LZO_MMODEL *in;
-unsigned char __LZO_MMODEL *out;
-lzo_uint in_len;
-lzo_uint out_len;
-
-	if ( !TestFlag(nffile->file_header->flags, FLAG_COMPRESSED) || block_header->id == CATALOG_BLOCK  ) {
-		ret =  write(nffile->fd, (void *)block_header, sizeof(data_block_header_t) + block_header->size);
-		if ( ret > 0 ) {
-			nffile->file_header->NumBlocks++;
-		}
-		return ret;
-	} 
-
-	out_block_header = (data_block_header_t *)lzo_buff;
-	*out_block_header = *(block_header);
-
-	in  = (unsigned char __LZO_MMODEL *)((pointer_addr_t)block_header     + sizeof(data_block_header_t));	
-	out = (unsigned char __LZO_MMODEL *)((pointer_addr_t)out_block_header + sizeof(data_block_header_t));	
-	in_len = block_header->size;
-	r = lzo1x_1_compress(in,in_len,out,&out_len,wrkmem);
-
-	if (r != LZO_E_OK) {
-		snprintf(error_string, ERR_SIZE,"compression failed: %d" , r);
-		error_string[ERR_SIZE-1] = 0;
-		return -2;
-	}
-
-	out_block_header->size = out_len;
-	ret =  write(nffile->fd, (void *)out_block_header, sizeof(data_block_header_t) + out_block_header->size);
-	if ( ret > 0 ) {
-		nffile->file_header->NumBlocks++;
-	}
-
-	return ret;
-
-} // End of WriteExtraBlock
-
-
-inline void ExpandRecord_v1(common_record_t *input_record, master_record_t *output_record ) {
-uint32_t	*u;
-size_t		size;
-void		*p = (void *)input_record;
-
-	// Copy common data block
-	size = sizeof(common_record_t) - sizeof(uint8_t[4]);
-	memcpy((void *)output_record, p, size);
-	p = (void *)input_record->data;
-
-	if ( (input_record->flags & FLAG_IPV6_ADDR) != 0 )	{ // IPv6
-		// IPv6
-		// keep compiler happy
-		// memcpy((void *)output_record->v6.srcaddr, p, 4 * sizeof(uint64_t));	
-		memcpy((void *)output_record->ip_union._ip_64.addr, p, 4 * sizeof(uint64_t));	
-		p = (void *)((pointer_addr_t)p + 4 * sizeof(uint64_t));
-	} else { 	
-		// IPv4
-		u = (uint32_t *)p;
-		output_record->v6.srcaddr[0] = 0;
-		output_record->v6.srcaddr[1] = 0;
-		output_record->v4.srcaddr 	 = u[0];
-
-		output_record->v6.dstaddr[0] = 0;
-		output_record->v6.dstaddr[1] = 0;
-		output_record->v4.dstaddr 	 = u[1];
-		p = (void *)((pointer_addr_t)p + 2 * sizeof(uint32_t));
-	}
-
-	// packet counter
-	if ( (input_record->flags & FLAG_PKG_64 ) != 0 ) { 
-		// 64bit packet counter
-		value64_t	l, *v = (value64_t *)p;
-		l.val.val32[0] = v->val.val32[0];
-		l.val.val32[1] = v->val.val32[1];
-		output_record->dPkts = l.val.val64;
-		p = (void *)((pointer_addr_t)p + sizeof(uint64_t));
-	} else {	
-		// 32bit packet counter
-		output_record->dPkts = *((uint32_t *)p);
-		p = (void *)((pointer_addr_t)p + sizeof(uint32_t));
-	}
-
-	// byte counter
-	if ( (input_record->flags & FLAG_BYTES_64 ) != 0 ) { 
-		// 64bit byte counter
-		value64_t	l, *v = (value64_t *)p;
-		l.val.val32[0] = v->val.val32[0];
-		l.val.val32[1] = v->val.val32[1];
-		output_record->dOctets = l.val.val64;
-		p = (void *)((pointer_addr_t)p + sizeof(uint64_t));
-	} else {	
-		// 32bit bytes counter
-		output_record->dOctets = *((uint32_t *)p);
-		p = (void *)((pointer_addr_t)p + sizeof(uint32_t));
-	}
-
-} // End of ExpandRecord_v1
-
-void UnCompressFile(char * filename) {
-int 			i, compressed, anonymized;
+void ModifyCompressFile(char * rfile, char *Rfile, int compress) {
+int 			i, anonymized, compression;
 ssize_t			ret;
 nffile_t		*nffile_r, *nffile_w;
 stat_record_t	*_s;
-char 			outfile[MAXPATHLEN];
-void			*tmp;
+char 			*filename, outfile[MAXPATHLEN];
 
-	nffile_r = OpenFile(filename, NULL);
-	if ( !nffile_r ) {
-		return;
-	}
+	SetupInputFileSequence(NULL, rfile, Rfile);
+
+	nffile_r = NULL;
+	while (1) {
+		nffile_r = GetNextFile(nffile_r, 0, 0);
+
+		// last file
+		if ( nffile_r == EMPTY_LIST )
+			break;
+
+		filename = GetCurrentFilename();
+
+		if ( !nffile_r || !filename) {
+			break;
+		}
 	
-	// tmp filename for new output file
-	snprintf(outfile, MAXPATHLEN, "%s-tmp", filename);
-	outfile[MAXPATHLEN-1] = '\0';
+		compression = FILE_COMPRESSION(nffile_r);
+		if ( compression == compress ) {
+			printf("File %s is already same compression methode\n", filename);
+			continue;
+		}
 
-	if ( FILE_IS_COMPRESSED(nffile_r) ) {
-		printf("Uncompress file %s ..\n", filename);
-		compressed = 0;
-	} else {
-		printf("Compress file %s .. \n", filename);
-		compressed = 1;
-	}
-	anonymized = IP_ANONYMIZED(nffile_r);
+		// tmp filename for new output file
+		snprintf(outfile, MAXPATHLEN, "%s-tmp", filename);
+		outfile[MAXPATHLEN-1] = '\0';
 
-	// allocate output file
-	nffile_w = OpenNewFile(outfile, NULL, compressed, anonymized, NULL);
-	if ( !nffile_w ) {
-		CloseFile(nffile_r);
-		DisposeFile(nffile_r);
-		return;
-	}
+		anonymized = IP_ANONYMIZED(nffile_r);
 
-	// Use same buffer for read/write
-	tmp = nffile_r->block_header;
-	nffile_r->block_header =  nffile_w->block_header;
-	nffile_r->buff_ptr 	   =  nffile_w->buff_ptr;
-
-	// swap stat records :)
-	_s = nffile_r->stat_record;
-	nffile_r->stat_record = nffile_w->stat_record;
-	nffile_w->stat_record = _s;
-
-	for ( i=0; i < nffile_r->file_header->NumBlocks; i++ ) {
-		ret = ReadBlock(nffile_r);
-		if ( ret < 0 ) {
-			LogError("Error while reading data block. Abort.\n");
-			nffile_r->block_header = tmp;
+		// allocate output file
+		nffile_w = OpenNewFile(outfile, NULL, compress, anonymized, NULL);
+		if ( !nffile_w ) {
 			CloseFile(nffile_r);
 			DisposeFile(nffile_r);
-			CloseFile(nffile_w);
-			DisposeFile(nffile_w);
-			unlink(outfile);
-			return;
+			break;;
 		}
-		if ( WriteBlock(nffile_w) <= 0 ) {
-			LogError("Failed to write output buffer to disk: '%s'" , strerror(errno));
-			nffile_r->block_header = tmp;
-			CloseFile(nffile_r);
-			DisposeFile(nffile_r);
-			CloseFile(nffile_w);
-			DisposeFile(nffile_w);
-			unlink(outfile);
-			return;
+
+		// swap stat records :)
+		_s = nffile_r->stat_record;
+		nffile_r->stat_record = nffile_w->stat_record;
+		nffile_w->stat_record = _s;
+	
+		for ( i=0; i < nffile_r->file_header->NumBlocks; i++ ) {
+			ret = ReadBlock(nffile_r);
+			if ( ret < 0 ) {
+				LogError("Error while reading data block. Abort.\n");
+				CloseFile(nffile_r);
+				DisposeFile(nffile_r);
+				CloseFile(nffile_w);
+				DisposeFile(nffile_w);
+				unlink(outfile);
+				return;
+			}
+
+			// swap buffers
+			void *_tmp = nffile_r->buff_pool[0];
+			nffile_r->buff_pool[0] = nffile_w->buff_pool[0];
+			nffile_w->buff_pool[0] = _tmp;
+			nffile_w->block_header = nffile_w->buff_pool[0];
+			nffile_r->block_header = nffile_r->buff_pool[0];
+			nffile_r->buff_ptr = (void *)((pointer_addr_t)nffile_r->block_header + sizeof(data_block_header_t));
+
+			if ( WriteBlock(nffile_w) <= 0 ) {
+				LogError("Failed to write output buffer to disk: '%s'" , strerror(errno));
+				CloseFile(nffile_r);
+				DisposeFile(nffile_r);
+				CloseFile(nffile_w);
+				DisposeFile(nffile_w);
+				unlink(outfile);
+				return;
+			}
 		}
+
+		printf("File %s compression changed\n", filename);
+		if ( !CloseUpdateFile(nffile_w, nffile_r->file_header->ident) ) {
+			unlink(outfile);
+			LogError("Failed to close file: '%s'" , strerror(errno));
+		} else {
+			unlink(filename);
+			rename(outfile, filename);
+		}
+
+		DisposeFile(nffile_w);
 	}
 
-	// file processed 
-	nffile_r->block_header = tmp;
-	CloseFile(nffile_r);
-
-	if ( !CloseUpdateFile(nffile_w, nffile_r->file_header->ident) ) {
-		unlink(outfile);
-		LogError("Failed to close file: '%s'" , strerror(errno));
-	} else {
-		unlink(filename);
-		rename(outfile, filename);
-	}
-
-	DisposeFile(nffile_r);
-	DisposeFile(nffile_w);
-
-} // End of UnCompressFile
+} // End of ModifyCompressFile
 
 void QueryFile(char *filename) {
 int i;
 nffile_t	*nffile;
-uint32_t num_records, type1, type2, type3;
+uint32_t num_records, type1, type2;
 struct stat stat_buf;
 ssize_t	ret;
 off_t	fsize;
@@ -1126,9 +1291,13 @@ off_t	fsize;
 	fsize = lseek(nffile->fd, 0, SEEK_CUR);
 	type1 = 0;
 	type2 = 0;
-	type3 = 0;
 	printf("File    : %s\n", filename);
-	printf("Version : %u - %s\n", nffile->file_header->version, FILE_IS_COMPRESSED(nffile) ? "compressed" : "not compressed");
+	printf ("Version : %u - %s\n", nffile->file_header->version,
+		FILE_IS_LZO_COMPRESSED (nffile) ? "lzo compressed" :
+		FILE_IS_LZ4_COMPRESSED (nffile) ? "lz4 compressed" :
+		FILE_IS_BZ2_COMPRESSED (nffile) ? "bz2 compressed" :
+            "not compressed");
+
 	printf("Blocks  : %u\n", nffile->file_header->NumBlocks);
 	for ( i=0; i < nffile->file_header->NumBlocks; i++ ) {
 		if ( (fsize + sizeof(data_block_header_t)) > stat_buf.st_size ) {
@@ -1161,9 +1330,6 @@ off_t	fsize;
 			case DATA_BLOCK_TYPE_2:
 				type2++;
 				break;
-			case Large_BLOCK_Type:
-				type3++;
-				break;
 			default:
 				printf("block %i has unknown type %u\n", i, nffile->block_header->id);
 		}
@@ -1191,7 +1357,6 @@ off_t	fsize;
 
 	printf(" Type 1 : %u\n", type1);
 	printf(" Type 2 : %u\n", type2);
-	printf(" Type 3 : %u\n", type3);
 	printf("Records : %u\n", num_records);
 
 	CloseFile(nffile);
@@ -1240,137 +1405,3 @@ int fd, ret;
 	return stat_record;
 
 } // End of GetStatRecord
-
-
-
-#ifdef COMPAT15
-/*
- * v1 -> v2 record conversion:
- * A netflow record in v1 block format has the same size as in v2 block format.
- * Therefore, the conversion rearranges the v1 layout into v2 layout
- *
- * old record size = new record size = 36bytes + x, where x is the sum of
- * IP address block (IPv4 or IPv6) + packet counter + byte counter ( 4/8 bytes) 
- *
- * v1											v2
- * 
- *  0 uint32_t    flags;						uint16_t	type; 	
- *												uint16_t	size;
- *
- *  1 uint16_t    size;							uint8_t		flags;		
- * 												uint8_t 	exporter_sysid;
- *    uint16_t    exporter_ref; => 0			uint16_t	ext_map;
- *
- *  2 uint16_t    msec_first;					uint16_t	msec_first;
- *    uint16_t    msec_last;					uint16_t	msec_last;
- *
- *  3 uint32_t    first;						uint32_t	first;
- *  4 uint32_t    last;							uint32_t	last;
- *
- *  5 uint8_t     dir;							uint8_t		fwd_status;
- *    uint8_t     tcp_flags;					uint8_t		tcp_flags;
- *    uint8_t     prot;							uint8_t		prot;
- *    uint8_t     tos;							uint8_t		tos;
- *
- *  6 uint16_t    input;						uint16_t	srcport;
- *    uint16_t    output;						uint16_t	dstport;
- *
- *  7 uint16_t    srcport;						x bytes IP/pkts/bytes
- *    uint16_t    dstport;
- *
- *  8 uint16_t    srcas;
- *    uint16_t    dstas;
- *												uint16_t    input;
- *												uint16_t    output;
- *
- *												uint16_t    srcas;
- *	9 x bytes IP/pkts/byte						uint16_t    dstas;
- *
- *
- */
-
-void Convert_v1_to_v2(void *mem) {
-common_record_t    *v2 = (common_record_t *)mem;
-common_record_v1_t *v1 = (common_record_v1_t *)mem;
-uint32_t *index 	   = (uint32_t *)mem;
-uint16_t tmp1, tmp2, srcas, dstas, *tmp3;
-size_t cplen;
-
-	// index 0
-	tmp1 	 = v1->flags;
-	v2->type = CommonRecordType;
-	v2->size = v1->size;
-
-	// index 1
-	v2->flags 		   = tmp1;
-	v2->exporter_sysid = 0;
-	v2->ext_map 	   = 0;
-
-	// index 2, 3, 4 already in sync
-
-	// index 5
-	v2->fwd_status = 0;
-
-	// index 6
-	tmp1 = v1->input;
-	tmp2 = v1->output;
-	v2->srcport = v1->srcport;
-	v2->dstport = v1->dstport;
-
-	// save AS numbers
-	srcas = v1->srcas;
-	dstas = v1->dstas;
-
-	cplen = 0;
-	switch (v2->flags) {
-		case 0:
-			// IPv4 8 byte + 2 x 4 byte counter
-			cplen = 16;
-			break;
-		case 1:
-			// IPv6 32 byte + 2 x 4 byte counter
-			cplen = 40;
-			break;
-		case 2:
-			// IPv4 8 byte + 1 x 4 + 1 x 8 byte counter
-			cplen = 20;
-			break;
-		case 3:
-			// IPv6 32 byte + 1 x 4 + 1 x 8 byte counter
-			cplen = 44;
-			break;
-		case 4:
-			// IPv4 8 byte + 1 x 8 + 1 x 4 byte counter
-			cplen = 20;
-			break;
-		case 5:
-			// IPv6 32 byte + 1 x 8 + 1 x 4 byte counter
-			cplen = 44;
-			break;
-		case 6:
-			// IPv4 8 byte + 2 x 8 byte counter
-			cplen = 24;
-			break;
-		case 7:
-			// IPv6 32 byte + 2 x 8 byte counter
-			cplen = 48;
-			break;
-		default:
-			// this should never happen - catch it anyway
-			cplen = 0;
-	}
-	// copy IP/pkts/bytes block
-	memcpy((void *)&index[7], (void *)&index[9], cplen );
-
-	// hook 16 bit array at the end of copied block
-	tmp3 = (uint16_t *)&index[7+(cplen>>2)];
-	// 2 byte I/O interfaces 
-	tmp3[0] = tmp1;
-	tmp3[1] = tmp2;
-	// AS numbers
-	tmp3[2] = srcas;
-	tmp3[3] = dstas;
-
-} // End of Convert_v1_to_v2
-#endif
-
